// A simple macro used to estimate the impact of possible EUSO rotations around X and Y axis within
// the cone defined by Beta aperture angle on the direction, Hmax, and energy.
// Author: Dmitry V.Naumov
// Date:   15 November, 2003
// Use: root[].L TildedEUSO.C
//      root[] FillTree(Nevents) // where Nevents is the number of with different Theta, Phi, Beta
//      root[] ReadTree()
void FillTree(int evMax)
{
  double Theta, Phi,ErrorTheta,ErrorPhi,Alpha, Beta;
  double ThetaDeg, PhiDeg,ErrorThetaDeg,ErrorPhiDeg, AlphaDeg, BetaDeg, Dir, ErrorH,ErrorEnergy;
  double deg=TMath::RadToDeg(), BetaMax=5*TMath::DegToRad();
  double AlphaX,AlphaY;
  int Events(0);
  TRotation r;
  TVector3 n(0,0,1),tilted,Omega,OmegaPrime;
  TMath math;
  // Creating the tree
  TTree tree("tree","Tildet EUSO");
  TFile f("output/tilded_euso.root","recreate");
  tree.Branch("Alpha",&AlphaDeg,"AlphaDeg/D");
  tree.Branch("Beta",&BetaDeg,"BetaDeg/D");
  tree.Branch("Theta",&ThetaDeg,"ThetaDeg/D");
  tree.Branch("Phi",&PhiDeg,"PhiDeg/D");
  tree.Branch("ErrorTheta",&ErrorThetaDeg,"ThetaDeg/D");
  tree.Branch("ErrorPhi",&ErrorPhiDeg,"PhiDeg/D");
  tree.Branch("Dir",&Dir,"Dir/D");
  tree.Branch("ErrorH",&ErrorH,"ErrorH/D");
  tree.Branch("ErrorEnergy",&ErrorEnergy,"ErrorEnergy/D");
  
  while(Events<evMax){
    Theta  = math.PiOver2()*gRandom->Rndm();
    Phi    = math.TwoPi()*gRandom->Rndm();
    Omega.SetXYZ(sin(Theta)*cos(Phi),sin(Theta)*sin(Phi),cos(Theta));
    Beta = BetaMax*gRandom->Rndm();    
    Events++;
    for(int i=0;i<1000;i++) {
      Alpha = math.TwoPi()*gRandom->Rndm();
      AlphaX = acos(sqrt(pow(sin(Beta),2)*pow(sin(Alpha),2)+pow(cos(Beta),2)));
      AlphaY = asin(sin(Beta)*cos(Alpha));
      r.RotateY(AlphaY);
      r.RotateX(-AlphaX);
      OmegaPrime = r*Omega;
      Dir        = Omega.Angle(OmegaPrime)*deg;
      ErrorTheta = OmegaPrime.Theta()-Theta;
      ErrorPhi   = OmegaPrime.Phi()-Phi;
      AlphaDeg   = Alpha*deg;
      BetaDeg       = Beta*deg;
      ThetaDeg      = Theta*deg;
      PhiDeg        = Phi*deg;
      ErrorThetaDeg = ErrorTheta*deg;
      ErrorPhiDeg   = ErrorPhi*deg;
      ErrorH        = OmegaPrime.CosTheta()/cos(Theta)-1;
      ErrorEnergy   = pow(cos(math.Pi()/6+Beta)/cos(TMath::Pi()/6),3)-1; // Maximum Error for 30 degrees
      tree.Fill();
      r.SetToIdentity();
    }
  }
  tree.Write();
  f.Close();
}
void ReadTree()
{
  TFile *f = new TFile("output/tilded_euso.root","");
  TTree *tree = (TTree*)f->Get("tree");
  double Alpha, Beta, Theta, Phi,ErrorTheta,ErrorPhi,Dir, ErrorH,ErrorEnergy;
  tree->SetBranchAddress("Alpha",&Alpha);
  tree->SetBranchAddress("Beta",&Beta);
  tree->SetBranchAddress("Theta",&Theta);
  tree->SetBranchAddress("Phi",&Phi);
  tree->SetBranchAddress("ErrorTheta",&ErrorTheta);
  tree->SetBranchAddress("ErrorPhi",&ErrorPhi);
  tree->SetBranchAddress("Dir",&Dir);
  tree->SetBranchAddress("ErrorH",&ErrorH);
  tree->SetBranchAddress("ErrorEnergy",&ErrorEnergy);
  tree->StartViewer();
  TH2F *t1 = new TH2F("t1","Altitude Relative Error",5,0,5,100,-1,1);
  TH2F *t2 = new TH2F("t2","Theta Error (deg)",5,0,5,100,-7,7);
  TH2F *t3 = new TH2F("t3","Phi Error (deg)",5,0,5,100,-20,20);
  TH2F *t4 = new TH2F("t4","Direction Error (deg)",5,0,5,100,0,10);
  tree->Draw("ErrorH:Beta>>t1","","");
  t1->Draw();
  c1->Print("output/ErrorH.gif");
  t1->FitSlicesY();
  t1_2->Draw();
  c1->Print("output/ErrorHSigma.gif");
  tree->Draw("ErrorEnergy:Beta");                     c1->Print("output/ErrorEnergy.gif");
  tree->Draw("ErrorTheta:Beta>>t2","",""); 
  t2->Draw();
  c1->Print("output/ErrorTheta.gif");
  t2->FitSlicesY();
  t2_2->Draw();
  c1->Print("output/ErrorThetaSigma.gif");
  tree->Draw("ErrorPhi:Beta>>t3","","");
  t3->Draw();
  c1->Print("output/ErrorPhi.gif");
  t3->FitSlicesY();
  t3_2->Draw();
  c1->Print("output/ErrorPhiSigma.gif");
  tree->Draw("Dir:Beta>>t4","",""); 
  t4->Draw();
  c1->Print("output/Dir.gif");
  t4->FitSlicesY();
  t4_2->Draw();
  c1->Print("output/DirSigma.gif");
}

void TildedEuso(int evMax)
{
  FillTree(evMax);
  ReadTree();
}
