// ESAF : Euso Simulation and Analysis Framework
// $Id: GlobalHisto.C,v 1.3 2005/01/06 00:24:56 thea Exp $
// A.Thea created May,  8 2003

#include "GlobalHisto.h"
#if !defined (__CINT__) || defined (__MAKECINT__)
#include "TMath.h"
#include "TArrayF.h"
#include "TTree.h"
#include "TBranch.h"
#include "TVector3.h"
#include "TH1.h"
#include "TH2.h"
#include "TPad.h"
#include "TLatex.h"

#include "iostream.h"
#include "EEvent.hh"


#endif

// constructor
GlobalHisto::GlobalHisto( TTree *tree ):
fEvent(0), fCutMinEnergy(0), fCutMaxEnergy(0), fCutMinThetaOne(0),
    fCutMaxThetaOne(0), fCutMinTheta(0), fCutMaxTheta(0) {

    Init(tree);
    SetXVariable("Energy");
}

void GlobalHisto::Init( TTree *tree ){

    fTree = tree;
    fTree->SetMakeClass(1);
    Int_t fNumPhotons;
    Float_t fTruth_fTrueEnergy;
    Float_t fTruth_fTrueShowerMaxPos[3];
    Float_t fTruth_fTrueTheta;

    // Getting branches
    TBranch *b_events = fTree->GetBranch("events");
    TBranch *b_fNumPhotons = fTree->GetBranch("fNumPhotons");

    TBranch *b_fTruth_fTrueEnergy = fTree->GetBranch("fTruth.fTrueEnergy");
    TBranch *b_fTruth_fTrueShowerMaxPos = fTree->GetBranch("fTruth.fTrueShowerMaxPos[3]");
    TBranch *b_fTruth_fTrueTheta = fTree->GetBranch("fTruth.fTrueTheta");

    fNumEvents = (Int_t) fTree->GetEntries();
   
    cout << "Read " << fNumEvents << " Events" << endl;
    
    fGlobalNumPhotons = new Int_t[fNumEvents];
    fGlobalTrueEnergy = new Float_t[fNumEvents];
    fGlobalTrueThetaOne = new Float_t[fNumEvents];
    fGlobalTrueTheta = new Float_t[fNumEvents];
    
    // Setting Addresses
    b_events->SetAddress(&fEvent);

    b_fNumPhotons->SetAddress(&fNumPhotons);
    b_fTruth_fTrueEnergy->SetAddress(&fTruth_fTrueEnergy); 
    b_fTruth_fTrueShowerMaxPos->SetAddress(fTruth_fTrueShowerMaxPos); 
    b_fTruth_fTrueTheta->SetAddress(&fTruth_fTrueTheta); 


    fGlobalMaxNumPhotons = 0;   

    fNoPhotons = 0;

    TVector3 TrueShowerMaxPos;
    for ( Int_t i=0; i < fNumEvents  ; i++) {

	b_fNumPhotons->GetEntry(i);

	if ( fNumPhotons == 0 ) {
	    fNoPhotons++;
	    // continue;
	}
	b_fTruth_fTrueEnergy->GetEntry(i);
	b_fTruth_fTrueTheta->GetEntry(i);
	b_fTruth_fTrueShowerMaxPos->GetEntry(i);

	fGlobalNumPhotons[i] = fNumPhotons;
	fGlobalTrueEnergy[i] = fTruth_fTrueEnergy/1e18;
	// FIXME: 
	TrueShowerMaxPos.SetXYZ( fTruth_fTrueShowerMaxPos[0],
	   fTruth_fTrueShowerMaxPos[1], 6371+430-fTruth_fTrueShowerMaxPos[2] );
	fGlobalTrueThetaOne[i] = TrueShowerMaxPos.Theta(); // *360/TMath::TwoPi();
	fGlobalTrueTheta[i] = fTruth_fTrueTheta; // *360/TMath::TwoPi();

    }

    fGlobalMaxNumPhotons = 
	fGlobalNumPhotons[TMath::LocMax(fNumEvents,fGlobalNumPhotons)];
}

// destructor
GlobalHisto::~GlobalHisto() {
    delete fGlobalNumPhotons;
    delete fGlobalTrueEnergy;
    delete fGlobalTrueThetaOne;
    delete fGlobalTrueTheta;
}

Bool_t GlobalHisto::IsInsideCuts(Int_t i) {
    if ( i < 0 || i > fNumEvents ) return kFALSE;
    Bool_t isInside(kTRUE);
    isInside = fGlobalNumPhotons[i] > 0;
    if ( fCutMinEnergy || fCutMaxEnergy )
       	 isInside &= ( fGlobalTrueEnergy[i] > fCutMinEnergy && 
	   fGlobalTrueEnergy[i] < fCutMaxEnergy);
    if ( fCutMinThetaOne || fCutMaxThetaOne ) 
	 isInside &= ( fGlobalTrueThetaOne[i] > fCutMinThetaOne &&
	   fGlobalTrueThetaOne[i] < fCutMaxThetaOne);
    if ( fCutMinTheta || fCutMaxTheta ) 
	isInside &= ( fGlobalTrueThetaOne[i] > fCutMinTheta &&
	   fGlobalTrueThetaOne[i] < fCutMaxTheta);  
    return isInside;
       
}

Bool_t GlobalHisto::SetEnergyCut(Float_t EMin, Float_t EMax) {
    if ( EMin > EMax ) return kFALSE;
    fCutMinEnergy = EMin;
    fCutMaxEnergy = EMax;
    return kTRUE;
}

Bool_t GlobalHisto::SetThetaOneCut(Float_t T1Min, Float_t T1Max) {
    if ( T1Min > T1Max ) return kFALSE;
    fCutMinThetaOne = T1Min;
    fCutMaxThetaOne = T1Max;
    return kTRUE;
}

Bool_t GlobalHisto::SetThetaCut(Float_t TMin, Float_t TMax) {
    if ( TMin > TMax ) return kFALSE;
    fCutMinTheta = TMin;
    fCutMaxTheta = TMax;
    return kTRUE;
}

Bool_t GlobalHisto::SetXVariable(Option_t *opt){
    TString option(opt);
    Bool_t set(kFALSE);
    if ( set = !(option.CompareTo("Energy")) ){ 
	fXVariable = fGlobalTrueEnergy;
        fXLabel = "Energy [EeV]";
	fXTitle = "Energy";
    }
    else if ( set = !(option.CompareTo("ThetaOne")) ) { 
	fXVariable = fGlobalTrueThetaOne;
        fXLabel = "#theta_{1} [rad]";
        fXTitle = "#theta_{1}";
    }
    else if ( set = !(option.CompareTo("Theta")) ) { 
	fXVariable = fGlobalTrueTheta;
        fXLabel = "#theta [rad]";
        fXTitle = "#theta";
    }
    
    if (set) {
	fCurrentX = opt;
	fXMin = fXVariable[TMath::LocMin(fNumEvents, fXVariable)];
	fXMax = fXVariable[TMath::LocMax(fNumEvents, fXVariable)];
    
    }
    return set;
    
}

TH1 * GlobalHisto::Do1DX() {

   TString hName = "EusoHF_G1D_True";
   hName += fCurrentX;
   // Cleaning old histogram
   gDirectory->Delete(hName);
    
   Int_t binX(100);
   TH1F *th0=new TH1F( hName, fXTitle + " of the Showers",
	 binX, fXMin, fXMax);
   for ( Int_t i=0; i < fNumEvents; i++) {
           if (IsInsideCuts(i))
	   th0->Fill(fXVariable[i]);
	}    
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   //th0->SetXTitle("Energy [EeV]");
   th0->SetXTitle(fXLabel);
   th0->SetYTitle("Events");
   th0->Draw();
   
   return th0;
}

TH1 * GlobalHisto::Do1DNumPhotons() {

   TString hName = "EusoHF_G1D_NumPhotons";
   // Cleaning old histogram
   gDirectory->Delete(hName);
    
   
   Int_t binNP(100);
   Int_t minNP=fGlobalNumPhotons[TMath::LocMin(fNumEvents,fGlobalNumPhotons)];
   Int_t maxNP=fGlobalNumPhotons[TMath::LocMax(fNumEvents,fGlobalNumPhotons)];

   Int_t deltaNP = (Int_t)TMath::Ceil((Float_t)(maxNP-minNP)/
                    (Float_t)(binNP-2));
   maxNP += deltaNP;

   TH1F *th0=new TH1F( hName, "Number of #gamma of the Showers", binNP, minNP, maxNP);
   for ( Int_t i=0; i < fNumEvents; i++) {
           if (IsInsideCuts(i))
	   th0->Fill(fGlobalNumPhotons[i]);
	}    
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   //th0->SetXTitle("Energy [EeV]");
   th0->SetXTitle(fXLabel);
   th0->SetYTitle("Events");
   th0->Draw();
   
   return th0;
}


//
// Detected Photons by Electronics Threshold
//

TH1 * GlobalHisto::Do1DDetectedPhotonsFromChip(){

   // some histo variables
   Int_t binMC(100);
   TString hname = "EusoHF_G1D_MadeCount";
   // Cleaning old histogram
   gDirectory->Delete(hname);

   UChar_t *fPhotons_fMadeCount = new UChar_t[fGlobalMaxNumPhotons];

   // getting branches
   TBranch *b_fPhotons_fMadeCount = fTree->GetBranch("fPhotons.fMadeCount");

   //setting addresses
   b_fPhotons_fMadeCount->SetAddress(fPhotons_fMadeCount);

   Int_t *NumMadeCount = new Int_t[fNumEvents];
   Int_t minMC, maxMC;
  
   for ( Int_t i=0; i < fNumEvents ; i++) {

	NumMadeCount[i] = 0;
	b_fPhotons_fMadeCount->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fMadeCount[k]) NumMadeCount[i]++;
	    }
    }
    minMC = NumMadeCount[TMath::LocMin(fNumEvents, NumMadeCount)];
    maxMC = NumMadeCount[TMath::LocMax(fNumEvents, NumMadeCount)];

    Int_t deltaMC = (Int_t)TMath::Ceil((Float_t)(maxMC-minMC)/
                    (Float_t)(binMC-2));
    maxMC+=deltaMC;
    minMC-=deltaMC;
    TH1 *th0=new TH1F( hname, "Detected #gamma from Chips",
                        binMC, minMC, maxMC);
   for (Int_t i=0; i < fNumEvents; i++) {
	if (IsInsideCuts(i))
	    th0->Fill(NumMadeCount[i]);
   }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   th0->SetXTitle("Detected #gamma");
   th0->SetYTitle("Events");
   th0->Draw();
   
   delete NumMadeCount;
   delete fPhotons_fMadeCount;  
   
   return th0;
} 

//
// Detected Photons by Macrocell Trigger
//

TH1 * GlobalHisto::Do1DDetectedPhotonsFromMacrocells(){

   // some histo variables
   Int_t binFO(100);
   TString hname = "EusoHF_G1D_MadeFastOR";

   // Cleaning old histogram
   gDirectory->Delete(hname);

   UChar_t *fPhotons_fMadeFastOR = new UChar_t[fGlobalMaxNumPhotons];

   // getting branches
   TBranch *b_fPhotons_fMadeFastOR = fTree->GetBranch("fPhotons.fMadeFastOR");

   //setting addresses
   b_fPhotons_fMadeFastOR->SetAddress(fPhotons_fMadeFastOR);

   Int_t *NumMadeFastOR = new Int_t[fNumEvents];
   Int_t minFO, maxFO;
  
   for ( Int_t i=0; i < fNumEvents ; i++) {

	NumMadeFastOR[i] = 0;
	b_fPhotons_fMadeFastOR->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fMadeFastOR[k]) NumMadeFastOR[i]++;
	    }
    }
    minFO = NumMadeFastOR[TMath::LocMin(fNumEvents, NumMadeFastOR)];
    maxFO = NumMadeFastOR[TMath::LocMax(fNumEvents, NumMadeFastOR)];

    Int_t deltaFO = (Int_t)TMath::Ceil((Float_t)(maxFO-minFO)/
                    (Float_t)(binFO-2));
    maxFO+=deltaFO;
    minFO-=deltaFO;
    TH1 *th0=new TH1F( hname, "Detected #gamma from Macrocells",
                        binFO, minFO, maxFO);
   for (Int_t i=0; i < fNumEvents; i++) {
	if (IsInsideCuts(i))
	    th0->Fill(NumMadeFastOR[i]);
   }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   th0->SetXTitle("Detected #gamma");
   th0->SetYTitle("Events");
   th0->Draw();
   
   delete NumMadeFastOR;
   delete fPhotons_fMadeFastOR;  
   
   return th0;
} 


//
// Photons lost in the optics
//

TH1 * GlobalHisto::Do1DPhotonsLostInOptics(){

   // some histo variables
   Int_t binNotHit(100);

   // Cleaning old histogram
   gDirectory->Delete("EusoHF_G1D_NoHitOnIFS");

   UChar_t *fPhotons_fHitOnIFS = new UChar_t[fGlobalMaxNumPhotons];

   // getting branches
   TBranch *b_fPhotons_fHitOnIFS = fTree->GetBranch("fPhotons.fHitOnIFS");

   //setting addresses
   b_fPhotons_fHitOnIFS->SetAddress(fPhotons_fHitOnIFS);

   Int_t *NumNotHitOnIFS = new Int_t[fNumEvents];
   Int_t minNotHit, maxNotHit;

   Int_t NumHitOnIFS;
   for ( Int_t i=0; i < fNumEvents ; i++) {
	NumHitOnIFS = 0;
	b_fPhotons_fHitOnIFS->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fHitOnIFS[k]) NumHitOnIFS++;
	    }
	NumNotHitOnIFS[i] = fGlobalNumPhotons[i]-NumHitOnIFS;
    }
    minNotHit = NumNotHitOnIFS[TMath::LocMin(fNumEvents, NumNotHitOnIFS)];
    maxNotHit = NumNotHitOnIFS[TMath::LocMax(fNumEvents, NumNotHitOnIFS)];

    Int_t deltaNotHit = (Int_t)TMath::Ceil((Float_t)(maxNotHit-minNotHit)/
                    (Float_t)(binNotHit-2));
    maxNotHit+=deltaNotHit;
    minNotHit-=deltaNotHit;
    TH1 *th0=new TH1F( "EusoHF_G1D_NoHitOnIFS", "Lost #gamma in the optics",
                        binNotHit, minNotHit, maxNotHit);
    for (Int_t i=0; i < fNumEvents; i++) {
	if (IsInsideCuts(i))
	    th0->Fill(NumNotHitOnIFS[i]);
    }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   th0->SetXTitle("Lost #gamma");
   th0->SetYTitle("Events");
   th0->Draw();
   
   delete NumNotHitOnIFS;
   delete fPhotons_fHitOnIFS;  
   
   return th0;
}

//
// Photons lost on the focal surface 
//

TH1 * GlobalHisto::Do1DPhotonsLostOnFocalPlane(){

   // some histo variables
   Int_t binNotSig(100);

   // Cleaning old histogram
   gDirectory->Delete("EusoHF_G1D_NoMadeSignal");

   UChar_t *fPhotons_fHitOnIFS = new UChar_t[fGlobalMaxNumPhotons];
   UChar_t *fPhotons_fMadeSignal = new UChar_t[fGlobalMaxNumPhotons];

   // getting branches
   TBranch *b_fPhotons_fHitOnIFS = fTree->GetBranch("fPhotons.fHitOnIFS");
   TBranch *b_fPhotons_fMadeSignal = fTree->GetBranch("fPhotons.fMadeSignal");

   //setting addresses
   b_fPhotons_fHitOnIFS->SetAddress(fPhotons_fHitOnIFS);
   b_fPhotons_fMadeSignal->SetAddress(fPhotons_fMadeSignal);

   Int_t *NumNotMadeSignal = new Int_t[fNumEvents];
   Int_t minNotSig, maxNotSig;

   Int_t NumHitOnIFS;
   Int_t NumMadeSignal;
    for ( Int_t i=0; i < fNumEvents ; i++) {
	NumHitOnIFS = 0;
        NumMadeSignal = 0;
	b_fPhotons_fHitOnIFS->GetEntry(i);
	b_fPhotons_fMadeSignal->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fHitOnIFS[k]) NumHitOnIFS++;
	    if (fPhotons_fMadeSignal[k]) NumMadeSignal++;
	    }
	NumNotMadeSignal[i] = NumHitOnIFS-NumMadeSignal;
    }
    minNotSig = NumNotMadeSignal[TMath::LocMin(fNumEvents, NumNotMadeSignal)];
    maxNotSig = NumNotMadeSignal[TMath::LocMax(fNumEvents, NumNotMadeSignal)];

    Int_t deltaNotHit = (Int_t)TMath::Ceil((Float_t)(maxNotSig-minNotSig)/
                    (Float_t)(binNotSig-2));
    maxNotSig+=deltaNotHit;
    minNotSig-=deltaNotHit;
    TH1 *th0=new TH1F( "EusoHF_G1D_NoMadeSignal", "Lost #gamma on the focal plane",
                        binNotSig, minNotSig, maxNotSig);
    for (Int_t i=0; i < fNumEvents; i++) {
	if (IsInsideCuts(i))
	    th0->Fill(NumNotMadeSignal[i]);
    }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   th0->SetXTitle("Lost #gamma");
   th0->SetYTitle("Events");
   th0->Draw();
   
   delete NumNotMadeSignal;
   delete fPhotons_fHitOnIFS;  
   
   return th0;
}

//
// Correlation Detected Photons Vs X
//

TH1 * GlobalHisto::Do2DDetectedPhotonsVsX(){
    TString hName = "EusoHF_G2D_DetectedPhotonsVs";
    hName += fCurrentX;

    // some histo variables
    Int_t binMC(100), binX(100);

    // Cleaning old histograms
    gDirectory->Delete(hName);

    UChar_t *fPhotons_fMadeCount = new UChar_t[fGlobalMaxNumPhotons];

    // getting branches
    TBranch *b_fPhotons_fMadeCount = fTree->GetBranch("fPhotons.fMadeCount");

    // setting addresses  
    b_fPhotons_fMadeCount->SetAddress(fPhotons_fMadeCount);

    //Float_t *TrueEnergy = new Float_t[fNumEvents];
    Int_t *NumMadeCount = new Int_t[fNumEvents];
    Int_t minMC(1000000), maxMC(0);
    // cout << "before main loop" << endl; 
    for ( Int_t i=0; i < fNumEvents  ; i++) {
	// cout << "in the loop " << i << endl; 
	NumMadeCount[i] = 0;
	b_fPhotons_fMadeCount->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i] ; k++){
	    if (fPhotons_fMadeCount[k]) NumMadeCount[i]++;
	    }
    }
    minMC = NumMadeCount[TMath::LocMin(fNumEvents, NumMadeCount)];
    maxMC = NumMadeCount[TMath::LocMax(fNumEvents, NumMadeCount)];
    
    Int_t deltaMC = (Int_t)TMath::Ceil((maxMC-minMC)/(Float_t)(binMC-2));
    maxMC+=deltaMC;
    //minMC-=deltaMC;
    //Float_t deltaEn = (maxEn-minEn)/(Float_t)(binEn-2);
    //maxEn+=deltaEn;
    //minEn-=deltaEn;

    TH2F *th0=new TH2F( hName, 
	   TString("Detected photons vs ") + fXTitle, binX, 
	   fXMin, fXMax, binMC, minMC, maxMC);
    for (Int_t i=0; i < fNumEvents; i++) {
	   if (IsInsideCuts(i))
	th0->Fill(fXVariable[i],NumMadeCount[i]);
    }
    th0->SetMarkerStyle( 8 );
    th0->SetMarkerColor( kRed );
    //th0->SetStats(kFALSE);
    // th0->SetXTitle("Energy [EeV]");
    th0->SetXTitle(fXLabel);
    th0->SetYTitle("Detected #gamma");
	   
    th0->Draw("scat");
    delete NumMadeCount;
    delete fPhotons_fMadeCount;  
   
   return th0;
} 

//
// Correlation A vs X 
//             A = (Detected Photons)/Energy
//

TH1 * GlobalHisto::Do2DXCorrelation(){
   TString hName = "EusoHF_G2D_DetPhOverEVs";
   hName += fCurrentX;
    
   // some histo variables
   Int_t binA(100), binX(100);

   // Cleaning old histograms
   gDirectory->Delete(hName);

   UChar_t *fPhotons_fMadeCount = new UChar_t[fGlobalMaxNumPhotons];

   // getting branches
   TBranch *b_fPhotons_fMadeCount = fTree->GetBranch("fPhotons.fMadeCount");

   // setting addresses
   b_fPhotons_fMadeCount->SetAddress(fPhotons_fMadeCount);

   Int_t *NumMadeCount = new Int_t[fNumEvents];
   Float_t minA(0.), maxA(1.);
  

   for ( Int_t i=0; i < fNumEvents ; i++) {

	NumMadeCount[i] = 0;
	b_fPhotons_fMadeCount->GetEntry(i);
	for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fMadeCount[k]) NumMadeCount[i]++;
	    }
	if ( minA > (Float_t)NumMadeCount[i]/fGlobalTrueEnergy[i] ) 
	     minA = (Float_t)NumMadeCount[i]/fGlobalTrueEnergy[i];
	if ( maxA < (Float_t)NumMadeCount[i]/fGlobalTrueEnergy[i] ) 
	     maxA = (Float_t)NumMadeCount[i]/fGlobalTrueEnergy[i];
   }

   Float_t deltaA = (maxA-minA)/(Float_t)(binA-2);
   maxA+=deltaA;
   minA-=deltaA;
   //Float_t deltaTh = (maxTh-minTh)/(Float_t)(binTh-2);
   //maxTh+=deltaTh;
   //minTh-=deltaTh;
   
   TH2F *th0=new TH2F( hName, TString("(Detected #gamma)/E vs ")+fXTitle, binX, fXMin, fXMax, binA, minA, maxA);
   for (Int_t i=0; i < fNumEvents; i++) {
       if ( IsInsideCuts(i) )
   	th0->Fill(fXVariable[i],(Float_t)NumMadeCount[i]/fGlobalTrueEnergy[i]);
   }
   th0->SetMarkerStyle( 8 );
   th0->SetMarkerColor( kRed );
   //th0->SetStats(kFALSE);
   th0->SetXTitle(fXLabel);
   th0->SetYTitle("(Det. #gamma)/Energy [EeV]^{-1}");
	   
   th0->Draw("scat");
   delete NumMadeCount;
   delete fPhotons_fMadeCount;  
   
   return th0;
} 


//
// Histogram of the number of hit pixels
//

TH1 * GlobalHisto::Do1DHitPixels(){

   // some histo variables
   Int_t binHP(100);

   // Cleaning old histograms
   gDirectory->Delete("EusoHF_G1D_HitPixels");
  
   Int_t *fMacrocell_fNumPixels;
   Int_t fNumCells;
   
   // getting branches
   TBranch *b_fNumCells = fTree->GetBranch("fNumCells");
   TBranch *b_fMacrocell_fNumPixels = fTree->GetBranch("fMacrocell.fNumPixels");
   
   // setting addresses
   b_fNumCells->SetAddress(&fNumCells); 

   Int_t *HitPixels = new Int_t[fNumEvents];
   Int_t minHP, maxHP, maxNumCells(0);
   
   // retrieving maximum number of macrocells
   for ( Int_t i=0; i < fNumEvents  ; i++) {
	b_fNumCells->GetEntry(i);
	if (fNumCells > maxNumCells) maxNumCells = fNumCells; 
   }
   // setting up the fNumPixel branch
   fMacrocell_fNumPixels = new Int_t[maxNumCells];   
   b_fMacrocell_fNumPixels->SetAddress(fMacrocell_fNumPixels); 

   
   for ( Int_t i=0; i < fNumEvents  ; i++) {
       HitPixels[i]=0;
       b_fNumCells->GetEntry(i);
       b_fMacrocell_fNumPixels->GetEntry(i);
       for ( Int_t k=0; k < fNumCells; k++){
	    HitPixels[i]+=fMacrocell_fNumPixels[k];
	    }
   }
   
   minHP = HitPixels[TMath::LocMin(fNumEvents, HitPixels)];
   maxHP = HitPixels[TMath::LocMax(fNumEvents, HitPixels)];

   Int_t deltaHP = (Int_t)TMath::Ceil((maxHP-minHP)/(Float_t)(binHP-2));
   maxHP+=deltaHP;
   minHP-=deltaHP;

   TH1 *th0=new TH1F( "EusoHF_G1D_HitPixels", "Hit Pixels", binHP, minHP, maxHP);
    for (Int_t i=0; i < fNumEvents; i++) {
	if ( IsInsideCuts(i) )
	    th0->Fill(HitPixels[i]);
    }
   
    th0->SetLineColor(37);
    th0->SetFillColor(2);
    // th0->SetStats(kFALSE);
    th0->SetXTitle("Hit Pixels");
    th0->SetYTitle("Events");
    th0->Draw();
    delete HitPixels;
    delete fMacrocell_fNumPixels;
   
   return th0;
}

//
// Histogram of the number of hit Macrocells 
//

TH1 * GlobalHisto::Do1DHitMacroCells(){

   // some histo variables
   Int_t binMC(20);

   // Cleaning old histograms
   gDirectory->Delete("EusoHF_G1D_HitMacrocells");
  
   Int_t fNumCells;
   
   // getting branches
   TBranch *b_fNumCells = fTree->GetBranch("fNumCells");
   
   // setting addresses
   b_fNumCells->SetAddress(&fNumCells); 

   Int_t *HitMacrocells = new Int_t[fNumEvents];
   Int_t minMC, maxMC;
   
   for ( Int_t i=0; i < fNumEvents  ; i++) {
       b_fNumCells->GetEntry(i);
       HitMacrocells[i]=fNumCells;
   }
   
   minMC = HitMacrocells[TMath::LocMin(fNumEvents, HitMacrocells)];
   maxMC = HitMacrocells[TMath::LocMax(fNumEvents, HitMacrocells)];

   Int_t deltaMC=maxMC-minMC;
   if (binMC > deltaMC) {
     binMC=deltaMC+1;
     maxMC+=1;
   }
//   else {
//      maxMC+=deltaMC*0.1;
//      minMC-=deltaMC*0.1;
//   }
   TH1 *th0=new TH1F( "EusoHF_G1D_HitMacrocells", "Hit Macrocells", binMC, minMC, maxMC);
    for (Int_t i=0; i < fNumEvents; i++) {
	if ( IsInsideCuts(i) )
	    th0->Fill(HitMacrocells[i]);
    }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   // th0->SetStats(kFALSE);
   th0->SetXTitle("Hit Macrocells");
   th0->SetYTitle("Events");
   th0->Draw();
   delete HitMacrocells;
   
   return th0;
}

//
// Histogram of the number of Fee 
//

TH1 * GlobalHisto::Do1DNumOfFee(){

   // some histo variables
   Int_t binNFee(100);

   // Cleaning old histograms
   gDirectory->Delete("EusoHF_G1D_NumOfFee");
  
   Int_t fNumFee;
   
   // getting branches
   TBranch *b_fNumFee = fTree->GetBranch("fNumFee");
   
   // setting addresses
   b_fNumFee->SetAddress(&fNumFee); 

   Int_t *NumOfFee = new Int_t[fNumEvents];
   Int_t minNFee, maxNFee;
   
   for ( Int_t i=0; i < fNumEvents  ; i++) {
       b_fNumFee->GetEntry(i);
       NumOfFee[i]=fNumFee;
   }
   
   minNFee = NumOfFee[TMath::LocMin(fNumEvents, NumOfFee)];
   maxNFee = NumOfFee[TMath::LocMax(fNumEvents, NumOfFee)];

   Int_t deltaNFee = (Int_t)TMath::Ceil((maxNFee-minNFee)/
                                     (Float_t)(binNFee-2));
   maxNFee+=deltaNFee;
   minNFee-=deltaNFee;
   
   TH1 *th0=new TH1F( "EusoHF_G1D_NumOfFee", "Number of Fee", binNFee, minNFee, maxNFee);
    for (Int_t i=0; i < fNumEvents; i++) {
	if ( IsInsideCuts(i) )
	    th0->Fill(NumOfFee[i]);
    }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   // th0->SetStats(kFALSE);
   th0->SetXTitle("Number of Fee");
   th0->SetYTitle("Events");
   th0->Draw();
   delete NumOfFee;
   
   return th0;
}

//
// Histogram of the number of AFee 
//

TH1 * GlobalHisto::Do1DNumOfAFee(){

   // some histo variables
   Int_t binNAFee(100);

   // Cleaning old histograms
   gDirectory->Delete("EusoHF_G1D_NumOfAFee");
  
   Int_t fNumAFee;
   
   // getting branches
   TBranch *b_fNumAFee = fTree->GetBranch("fNumAFee");
   
   // setting addresses
   b_fNumAFee->SetAddress(&fNumAFee); 

   Int_t *NumOfAFee = new Int_t[fNumEvents];
   Int_t minNAFee, maxNAFee;
   
   for ( Int_t i=0; i < fNumEvents  ; i++) {
       b_fNumAFee->GetEntry(i);
       NumOfAFee[i]=fNumAFee;
   }
   
   minNAFee = NumOfAFee[TMath::LocMin(fNumEvents, NumOfAFee)];
   maxNAFee = NumOfAFee[TMath::LocMax(fNumEvents, NumOfAFee)];

   Int_t deltaNAFee = (Int_t)TMath::Ceil((maxNAFee-minNAFee)/
                                     (Float_t)(binNAFee-2));
   maxNAFee+=deltaNAFee;
   minNAFee-=deltaNAFee;
   
   TH1 *th0=new TH1F( "EusoHF_G1D_NumOfAFee", "Number of AFee", binNAFee, minNAFee, maxNAFee);
    for (Int_t i=0; i < fNumEvents; i++) {
	if ( IsInsideCuts(i) )
	    th0->Fill(NumOfAFee[i]);
    }
   
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   // th0->SetStats(kFALSE);
   th0->SetXTitle("Number of Fee");
   th0->SetYTitle("Events");
   th0->Draw();
   delete NumOfAFee;
   
   return th0;
}



//
// Histogram of the number of triggers versus X 
//

TH1 * GlobalHisto::Do1DTriggerVsX() {
   TString hName = "EusoHF_G1D_TriggerVs";
   hName += fCurrentX;

   // some histo variables
   Int_t binX(100);

   // Cleaning old histograms
   gDirectory->Delete(hName);
  
   Int_t *fMacrocell_fHasTriggered;
   Int_t fNumCells;
   
   // getting branches
   TBranch *b_fNumCells = fTree->GetBranch("fNumCells");
   TBranch *b_fMacrocell_fHasTriggered = fTree->GetBranch("fMacrocell.fHasTriggered");
   
   // setting addresses
   b_fNumCells->SetAddress(&fNumCells); 

   Int_t *HasTriggered = new Int_t[fNumEvents];
   Int_t maxNumCells(0);
   
   // retrieving maximum number of macrocells
   for ( Int_t i=0; i < fNumEvents  ; i++) {
	b_fNumCells->GetEntry(i);
	if (fNumCells > maxNumCells) maxNumCells = fNumCells; 
   }
   // setting up the fHasTriggered branch
   fMacrocell_fHasTriggered = new Int_t[maxNumCells];   
   b_fMacrocell_fHasTriggered->SetAddress(fMacrocell_fHasTriggered); 

   
   for ( Int_t i=0; i < fNumEvents  ; i++) {
       HasTriggered[i]=0;
       b_fNumCells->GetEntry(i);
       b_fMacrocell_fHasTriggered->GetEntry(i);
       for ( Int_t k=0; k < fNumCells; k++){
	    HasTriggered[i]+=fMacrocell_fHasTriggered[k];
	    }
   }
   //Float_t deltaEn = (maxEn-minEn)/(Float_t)(binEn-2);
   //maxEn+=deltaEn;
   //minEn-=deltaEn;

   TH1 *th0=new TH1F( hName, TString("Trigger vs ")+fXTitle, binX, fXMin, fXMax);
   TH1 *th1=new TH1F( hName+"_Temp", fXTitle, binX, fXMin, fXMax); 
   for (Int_t i=0; i < fNumEvents; i++) {
       if (IsInsideCuts(i)){
	   if ( HasTriggered[i] )
	      th0->Fill(fXVariable[i]);
	   th1->Fill(fXVariable[i]);
       }
   }
   
   th0->Divide(th1);
   th0->SetLineColor(37);
   th0->SetFillColor(2);
   //th0->SetStats(kFALSE);
   th0->SetXTitle(fXLabel);
   th0->SetYTitle("% trigger");
   th0->Draw();

   delete th1;
   delete HasTriggered;
   delete fMacrocell_fHasTriggered;
   
   return th0;
}


//
// Correlation Average (Num of Photons per Pixel)/X
//

TH1 * GlobalHisto::Do2DAveragePhotonVsX() {
   TString hName = "EusoHF_G1D_AveragePhotonVs";
   hName += fCurrentX;

   
   // some histo variables
   Int_t binAv(100), binX(100);

   // Cleaning old histograms
   gDirectory->Delete(hName);

   UChar_t *fPhotons_fMadeCount = new UChar_t[fGlobalMaxNumPhotons];
   Int_t *fMacrocell_fNumPixels;
   Int_t fNumCells;

   // getting branches
   TBranch *b_fPhotons_fMadeCount = fTree->GetBranch("fPhotons.fMadeCount");
   TBranch *b_fNumCells = fTree->GetBranch("fNumCells");
   TBranch *b_fMacrocell_fNumPixels = fTree->GetBranch("fMacrocell.fNumPixels");

   // setting addresses
   b_fNumCells->SetAddress(&fNumCells); 
   b_fPhotons_fMadeCount->SetAddress(fPhotons_fMadeCount);
   
   Int_t *NumMadeCount = new Int_t[fNumEvents];
   Int_t *HitPixels = new Int_t[fNumEvents];
   Float_t minAv(10000.), maxAv(0.);
   Int_t maxNumCells(0);
  
   // retrieving maximum number of macrocells
   for ( Int_t i=0; i < fNumEvents  ; i++) {
	b_fNumCells->GetEntry(i);
	if (fNumCells > maxNumCells) maxNumCells = fNumCells; 
   }
   // setting up the fNumPixel branch
   fMacrocell_fNumPixels = new Int_t[maxNumCells];   
   b_fMacrocell_fNumPixels->SetAddress(fMacrocell_fNumPixels); 


   for ( Int_t i=0; i < fNumEvents  ; i++) {

	NumMadeCount[i] = 0;
	HitPixels[i]=0;
		
	b_fNumCells->GetEntry(i);
	b_fPhotons_fMadeCount->GetEntry(i);
	b_fMacrocell_fNumPixels->GetEntry(i);
        for ( Int_t k=0; k < fGlobalNumPhotons[i]; k++){
	    if (fPhotons_fMadeCount[k]) NumMadeCount[i]++;
	    }
        for ( Int_t k=0; k < fNumCells; k++){
	    HitPixels[i]+=fMacrocell_fNumPixels[k];
	    }
	if (HitPixels[i] != 0){
	   if ( minAv > (Float_t)NumMadeCount[i]/(Float_t)HitPixels[i] )
		minAv = (Float_t)NumMadeCount[i]/(Float_t)HitPixels[i];
	   if ( maxAv < (Float_t)NumMadeCount[i]/(Float_t)HitPixels[i] )
		maxAv = (Float_t)NumMadeCount[i]/(Float_t)HitPixels[i];
	}
   }
   Float_t deltaAv = (maxAv-minAv)/(Float_t)(binAv-2);
   maxAv+=deltaAv;
   minAv-=deltaAv;
   //Float_t deltaAl = (maxAl-minAl)/(Float_t)(binAl-2);
   //maxAl+=deltaAl;
   //minAl-=deltaAl;
   
   TH2F *th0=new TH2F( hName, TString("Average #gamma per Pixel Vs " 
	       + fXTitle), binX, fXMin, fXMax, binAv, minAv, maxAv);
   for (Int_t i=0; i < fNumEvents; i++) {
	if (HitPixels[i] != 0 && IsInsideCuts(i) )
   	   th0->Fill(fXVariable[i],(Float_t)NumMadeCount[i]/(Float_t)HitPixels[i]);
   }
   th0->SetMarkerStyle( 8 );
   th0->SetMarkerColor( kRed );
   //th0->SetStats(kFALSE);
   th0->SetXTitle(fXLabel);
   th0->SetYTitle("(Det. #gamma)/N_{pixel}");
	   
   th0->Draw("scat");

   // Adding TLatex 
   // some operations to calculate text position
    
   Float_t frameHeight = th0->GetYaxis()->GetXmax()-
                         th0->GetYaxis()->GetXmin();
   Float_t frameWidth = th0->GetXaxis()->GetXmax()-
                         th0->GetXaxis()->GetXmin();
   Float_t textSize = 0.04;
   Float_t textSizeFactor = frameHeight/
		       (1-gPad->GetTopMargin()-gPad->GetBottomMargin());
   Float_t textTopBorder = th0->GetYaxis()->GetXmax()
                           -1.5*textSize*textSizeFactor;
   Float_t textLeftBorder = th0->GetXaxis()->GetXmin()+0.01*frameWidth;
    
   TLatex t;
   Char_t txt[200];
   t.SetTextFont( 72 );
   t.SetTextColor( 50 );
   t.SetTextSize( textSize );
   sprintf(txt,"N^{o} of ev. (HitPixel>0) : %d", (Int_t) th0->GetEntries());

   t.DrawLatex( textLeftBorder, textTopBorder, txt);
   
   //if (minEnergy != 0 || maxEnergy != 0) {
   //   sprintf( txt, "Energy cut: Min = %.2e, Max = %.2e",
   //	      minEnergy, maxEnergy);
   //     
   //   t.DrawLatex( textLeftBorder, 
   //          textTopBorder-textSize*textSizeFactor*1.1, txt);
   //  }
   
   delete NumMadeCount;
   delete HitPixels;
   delete fPhotons_fMadeCount;
   delete fMacrocell_fNumPixels;
   
   return th0;

} 

