// $Id: MakeHistoSingleEvent.C,v 1.1 2003/06/12 22:15:31 thea Exp $
// M. Pallavicini / A. Thea created 16/04/03
// root Macro designed to produce standard Euso Histograms
// Display information about a single event
// 
// 
// Histo Name                    Description
// GtuPhotons                 Photons reaching different simulation point
//                            vs time 
//                            

#include "TH1F.h"
#include "TChain.h"
#include "TKey.h"
#include "TText.h"

// this plot is the number of photons in different stage of the simulation
// as a function of the GTU number; the first photon defines the first GTU
// it is not exactly the GTU simulated by the macrocell because this number
// is not available for the photons that did not give signal into the
// front end electronics
// 3 stages: photons entering the pupil, reaching the FS and giving signal in PMT
// the gtu length is in micro-seconds
void DoGtuPhotonsHisto(Int_t event=0, TTree *tree=0, Float_t gtu_len=2.5) {
   if ( !tree ) {
      tree = (TTree*)gDirectory->FindObject("etree");
      if ( !tree) {
	 tree = (TTree*)gDirectory->GetKey("etree")->ReadObj();
         if ( !tree) {
            printf("Cannot find a valid Euso etree\n");
	    return;
         }
      }
   }
   
   // create and fill histograms 
   TH1F *th0 = (TH1F*)gROOT->FindObject("EventGtuPhotons0");
   if ( !th0 )
      th0 = new TH1F("EventGtuPhotons0","Photons vs GTU number",100,0.,100.);
   else
      th0->Reset();
   char cut[300],var[300];
   sprintf(cut,"fHeader.fNum == %d",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons0",gtu_len);
   tree->Draw(var,cut,"",1,event);
   
   TH1F *th1 = (TH1F*)gROOT->FindObject("EventGtuPhotons1");
   if ( !th1 )
      th1 = new TH1F("EventGtuPhotons1","Photons vs GTU number",100,0.,100.);
   else
      th1->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons1",gtu_len);
   tree->Draw(var,cut,"",1,event);
   
   TH1F *th2 = (TH1F*)gROOT->FindObject("EventGtuPhotons2");
   if ( !th2 )
      th2 = new TH1F("EventGtuPhotons2","Photons vs GTU number",100,0.,100.);
   else
      th2->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeSignal>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons2",gtu_len);
   tree->Draw(var,cut,"",1,event);

   // draw histograms with colors and filling
   th0->SetStats(0);
   th0->SetLineColor(9);
   th0->SetFillColor(0);
   th0->SetXTitle("GTU number");
   th0->SetYTitle("Number of Photons");
   th0->Draw();
   th1->SetStats(0);
   th1->SetLineColor(2);
   th1->SetFillColor(0);
   th1->Draw("same");
   th2->SetStats(0);
   th2->SetLineColor(8);
   th2->SetFillColor(8);
   th2->Draw("same");

   // write meaning and statistics
   TText *t = new TText();
   char txt[200];
   t->SetTextFont(62);
   t->SetTextColor(9);
   t->SetTextSize(0.04);
   sprintf(txt,"Ph. on Pupil:%d",(int)th0->GetEntries());
   t->DrawText(60.,th0->GetMaximum()*0.94,txt); 
   t->SetTextColor(2);
   sprintf(txt,"Ph. on FS:%d",(int)th1->GetEntries());
   t->DrawText(60.,th0->GetMaximum()*0.86,txt); 
   t->SetTextColor(8);
   sprintf(txt,"Ph. detected:%d",(int)th2->GetEntries());
   t->DrawText(60.,th0->GetMaximum()*0.78,txt);
}

// same as before starting from Photons reaching the ideal focal surface
// (in this plot the photons that entered the pupil and were lot in the
// optical system are not accounted for 
// the effect of single channel and digital counter threshold are highlighted 
void DoGtuPhotonsHisto2(Int_t event=0, TTree *tree=0, Float_t gtu_len=2.5) {
   if ( !tree ) {
      tree = (TTree*)gDirectory->FindObject("etree");
      if ( !tree) {
	 tree = (TTree*)gDirectory->GetKey("etree")->ReadObj();
         if ( !tree) {
            printf("Cannot find a valid Euso etree\n");
	    return;
         }
      }
   }
   
   TH1F *th1 = (TH1F*)gROOT->FindObject("EventGtuPhotons1");
   if ( !th1 )
      th1 = new TH1F("EventGtuPhotons1","Photons vs GTU number",100,0.,100.);
   else
      th1->Reset();
   
   char cut[300],var[300];
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons1",gtu_len);
   tree->Draw(var,cut,"",1,event);
   
   TH1F *th2 = (TH1F*)gROOT->FindObject("EventGtuPhotons2");
   if ( !th2 )
      th2 = new TH1F("EventGtuPhotons2","Photons vs GTU number",100,0.,100.);
   else
      th2->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeSignal>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons2",gtu_len);
   tree->Draw(var,cut,"",1,event);

   TH1F *th3 = (TH1F*)gROOT->FindObject("EventGtuPhotons3");
   if ( !th3 )
      th3 = new TH1F("EventGtuPhotons3","Photons vs GTU number",100,0.,100.);
   else
      th3->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeCount>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons3",gtu_len);
   tree->Draw(var,cut,"",1,event);

   TH1F *th4 = (TH1F*)gROOT->FindObject("EventGtuPhotons4");
   if ( !th4 )
      th4 = new TH1F("EventGtuPhotons4","Photons vs GTU number",100,0.,100.);
   else
      th4->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeFastOR>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons4",gtu_len);
   tree->Draw(var,cut,"",1,event);

   th1->SetStats(0);
   th1->SetLineColor(9);
   th1->SetFillColor(0);
   th1->SetXTitle("GTU number");
   th1->SetYTitle("Number of Photons");
   th1->Draw();
   th2->SetStats(0);
   th2->SetLineColor(6);
   th2->SetFillColor(0);
   th2->Draw("same");
   th3->SetStats(0);
   th3->SetLineColor(2);
   th3->SetFillColor(2);
   th3->Draw("same");
   th4->SetStats(0);
   th4->SetLineColor(8);
   th4->SetFillColor(8);
   th4->Draw("same");

   TText *t = new TText();
   char txt[100];
   t->SetTextFont(62);
   t->SetTextColor(9);
   t->SetTextSize(0.04);
   sprintf(txt,"Photons on FS:%d",(int)th1->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.94,txt); 
   t->SetTextColor(6);
   sprintf(txt,"Pmt signals:%d",(int)th2->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.86,txt); 
   t->SetTextColor(2);
   sprintf(txt,"Frontend hits:%d",(int)th3->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.78,txt);
   t->SetTextColor(8);
   sprintf(txt,"Fast OR counts:%d",(int)th4->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.70,txt);
}
/**
// same as before starting from Photons detected by PMTs
// (in this plot the photons that entered the pupil and were lot in the
// optical system or those that were not detected because of QE are not accounted for 
// the effect of single channel and digital counter threshold are highlighted
// Also AFEE infos are added here (FULL AFEE)
void DoGtuPhotonsHisto3(Int_t event=0, TTree *tree=0, Float_t gtu_len=2.5) {
   if ( !tree ) {
      tree = (TTree*)gDirectory->FindObject("etree");
      if ( !tree) {
	 tree = (TTree*)gDirectory->GetKey("etree")->ReadObj();
         if ( !tree) {
            printf("Cannot find a valid Euso etree\n");
	    return;
         }
      }
   }
   
   char cut[300],var[300];
   TH1F *th1 = (TH1F*)gROOT->FindObject("EventGtuPhotons1");
   if ( !th1 )
      th1 = new TH1F("EventGtuPhotons1","Photons vs GTU number",100,0.,100.);
   else
      th1->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeSignal>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons1",gtu_len);
   tree->Draw(var,cut,"",1,event);
   
   TH1F *th2 = (TH1F*)gROOT->FindObject("EventGtuPhotons2");
   if ( !th2 )
      th2 = new TH1F("EventGtuPhotons2","Photons vs GTU number",100,0.,100.);
   else
      th2->Reset();
   sprintf(cut,"fPhotons.fHitOnIFS>0 && fHeader.fNum == %d && fPhotons.fMadeCount>0.1",event);
   sprintf(var,"((fPhotons.fTime-fFirstTime)/(1000.*%f))>>EventGtuPhotons2",gtu_len);
   tree->Draw(var,cut,"",1,event);

   TH1F *th3 = (TH1F*)gROOT->FindObject("EventGtuPhotons3");
   if ( !th3 )
      th3 = new TH1F("EventGtuPhotons3","Photons vs GTU number",100,0.,100.);
   else
      th3->Reset();

   // compute charge for each GTU and fill histogram
   Float_t total_afee_charge=0.;
   TBranch *br = tree->GetBranch("events");   
   EEvent *pEv = new EEvent();
   br->SetAddress( &pEv );
   tree->GetEntry( event );
   
   for(Int_t i=0; i < pEv->GetNumAFee(); i++ ) {
       EAFee *afee = pEv->GetEAFee( i );
       Int_t gtu = afee->GetGtuId();
       Float_t charge = afee->GetDyCharge();
       Int_t numpe = charge*2.e13;   // convert into NOMINAL photoelectrons, gain 3.E5
       if ( gtu > 0 && gtu < 100 ) {
	  Int_t old = th3->GetBinContent( gtu );
          th3->SetBinContent( gtu, old + numpe );
	  total_afee_charge += numpe;
       }
   }

   th1->SetStats(0);
   th1->SetLineColor(9);
   th1->SetFillColor(0);
   th1->SetXTitle("GTU number");
   th1->SetYTitle("Number of Photons");
   th1->Draw();
   th2->SetStats(0);
   th2->SetLineColor(6);
   th2->SetFillColor(0);
   th2->Draw("same");
   th3->SetStats(0);
   th3->SetLineColor(8);
   th3->SetFillColor(8);
   th3->Draw("same");

   TText *t = new TText();
   char txt[100];
   t->SetTextFont(62);
   t->SetTextColor(9);
   t->SetTextSize(0.04);
   sprintf(txt,"Ph. detected:%d",th1->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.94.,txt); 
   t->SetTextColor(6);
   sprintf(txt,"Ph. in DFEE:%d",th2->GetEntries());
   t->DrawText(65.,th1->GetMaximum()*0.86,txt); 
   t->SetTextColor(2);
   sprintf(txt,"Charge (pC,dynode):%f",total_afee_charge);
   t->DrawText(65.,th1->GetMaximum()*0.78,txt);
}
**/

void MakeHistoSingleEvent(int histo_set=0, int event=0, TChain *chain=0) {      
}
