
void loadlibs() {
   gSystem->Load("libatmosphere.so");
   gSystem->Load("libgenbase.so");
   gSystem->Load("libshowers.so");
}

ShowerTrack* CheckEnergyDistribution() {
   gROOT->Reset();
   loadlibs(); 
   gStyle->SetOptStat(0);
   SlastShowerGenerator *ShowerGenerator = new SlastShowerGenerator;
   ShowerTrack *track = ShowerGenerator->Get();
   Int_t Nsteps = track->Size();
   if ( !Nsteps ) {
      cout << "The track is empty\n";
      return NULL;
   }
   TH1F* histos[10000];
   TCanvas *c1;
   if (c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_checkEnergy"))
       c1->Clear();
   else
       c1 = new TCanvas("c_checkEnergy");
   TLegend *legend = new TLegend(0.8,0.9,1,0.6);
   Int_t frames = Nsteps/10;
   Float_t NumMax(-1);
   for (Int_t i = 0; i< TMath::Min(Nsteps,10000); i ++) { 
       ShowerStep& s = track->GetStep(i);
       if (NumMax < s.GetNelectrons()) NumMax = s.GetNelectrons();
   }
   for (Int_t i = 0; i< Nsteps; i +=frames) {
       ShowerStep& s = track->GetStep(i);
       histos[i] = s.GetHistoEnergy();
       TString opt = "";
       if (i != 0) opt = "same";
       histos[i]->Draw(opt);
       histos[i]->SetLineColor(50 + (Int_t) (50.*s.GetNelectrons()/NumMax));
       legend->AddEntry(histos[i],Form("step n = %d age = %.2f\n",i,s.GetAgei()),"lp");
   }
   legend->Draw();
   histos[0]->SetMaximum(1.e-1);
   histos[0]->SetMinimum(1.e-3);
   histos[0]->SetXTitle("Energy, [MeV]");
   c1->SetLogy(kTRUE);
   c1->SetLogx(kTRUE);
   c1->cd(0);
   c1->Print("output/check_energydistributions.eps");
   return track;
}

ShowerTrack* CheckLateralDistribution() {
   loadlibs(); 
   gStyle->SetOptStat(0);
   SlastShowerGenerator *ShowerGenerator = new SlastShowerGenerator;
   ShowerTrack *track = ShowerGenerator->Get();
   Int_t Nsteps = track->Size();
   if ( !Nsteps ) {
      cout << "The track is empty\n";
      return NULL;
   }
   TH1F* histos[10000];
   TCanvas *c1;
   if (c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_checkLateral"))
       c1->Clear();
   else
       c1 = new TCanvas("c_checkLateral");
   TLegend *legend = new TLegend(0.8,0.9,1,0.6);
   Int_t frames = Nsteps/10;
   Float_t NumMax(-1);
   for (Int_t i = 0; i< TMath::Min(Nsteps,10000); i ++) { 
       ShowerStep& s = track->GetStep(i);
       if (NumMax < s.GetNelectrons()) NumMax = s.GetNelectrons();
   }
   for (Int_t i = 0; i< Nsteps; i +=frames) { 
       ShowerStep& s = track->GetStep(i);
       histos[i] = s.GetHistoLateral();
       TString opt = "";
       if (i != 0) opt = "same";
       histos[i]->Draw(opt);
       histos[i]->SetLineColor(50 + (Int_t) (50.*s.GetNelectrons()/NumMax));
       legend->AddEntry(histos[i],Form("step n = %d age = %.2f\n",i,s.GetAgei()),"lp");
   }
   legend->Draw();
   histos[0]->SetMaximum(1.e0);
   histos[0]->SetMinimum(1.e-10);
   c1->SetLogy(kTRUE);
   c1->Print("output/check_lateraldistributions.eps");

   return track;

}

//
void CheckSteps()  {
	loadlibs();
	gStyle->SetOptStat(0);
	CorsikaFileShowerGenerator *ShowerGenerator = new CorsikaFileShowerGenerator;
	ShowerTrack *track = ShowerGenerator->Get();
        Int_t Nsteps = track->Size();
	TH1F *h1 = new TH1F ("h1", "fStep.fXi", Nsteps, 0, Nsteps);
	TH1F *h2 = new TH1F ("h2", "fStep.fXYZi[0]", Nsteps, 0, Nsteps);
	TH1F *h3 = new TH1F ("h3", "fStep.fXYXi[1]", Nsteps, 0, Nsteps);
	TH1F *h4 = new TH1F ("h4", "fStep.fXYZi[2]", Nsteps, 0, Nsteps);
	TH1F *h5 = new TH1F ("h5", "fStep.fTimei", Nsteps, 0, Nsteps);
	TH1F *h6 = new TH1F ("h6", "fStep.fAgei", Nsteps, 0, Nsteps);
	TH1F *h7 = new TH1F ("h7", "fStep.fNelectrons", Nsteps, 0, Nsteps);
	TH1F *h8 = new TH1F ("h8", "fStep.fNcharged", Nsteps, 0, Nsteps);
	TH1F *h9 = new TH1F ("h9", "fStep.fEloss", Nsteps, 0, Nsteps);

   TCanvas *c1 = new TCanvas();
   Int_t icolor=3;
   c1->Divide(3,3);
   for (Int_t i = 0; i< Nsteps; i ++) {
         ShowerStep s = track->GetStep(i);
         h1->Fill(i,s.GetXi());
         h2->Fill(i,s.GetXYZi().X());
         h3->Fill(i,s.GetXYZi().Y());
         h4->Fill(i,s.GetXYZi().Z());
         h5->Fill(i,s.GetTimei());
         h6->Fill(i,s.GetAgei());
         h7->Fill(i,s.GetNelectrons());
         h8->Fill(i,s.GetNcharged());
         h9->Fill(i,s.GetEloss());
}

   c1->cd(1);
   h1->Draw();
   c1->cd(2);
   h2->Draw();
   c1->cd(3);
   h3->Draw();
   c1->cd(4);
   h4->Draw();
   c1->cd(5);
   h5->Draw();
   c1->cd(6);
   h6->Draw();
   c1->cd(7);
   h7->Draw();
   c1->cd(8);
   h8->Draw();
   c1->cd(9);
   h9->Draw();

   c1->Print("output/check_steps.eps");
}
