// $Id: LowtranManager.hh,v 1.4 2006/03/24 10:49:12 thea Exp $
// Author:    2006/03/15

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: LowtranManager                                                       *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __LOWTRANMANAGER_HH__
#define __LOWTRANMANAGER_HH__

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "EsafMsgSource.hh"
#include <TArrayF.h>
#include <TString.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// LowtranTransferOutput                                                      //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

struct LowtranTransferOutput {
    Float_t R;
    Float_t Altitude;
    Float_t Angle;
    Float_t WaveLength[100];
    Float_t Transmission[100];
    Float_t Ozone[100];
    Float_t Rayl[100];
    Float_t Aerosol[100];
};

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// LowtranManager                                                             //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class LowtranManager : public EsafConfigurable, public EsafMsgSource {
public:
    virtual ~LowtranManager();

    
    Bool_t InitLowtran();
    Bool_t LwTrans(Int_t init, Float_t altitude, Float_t angle, Float_t *trans);
    Bool_t LwTrans_V2(Int_t init, Float_t h1, Float_t h2, Float_t theta);

    Float_t GetR() const { return fR; }
    Float_t GetAltitude() const { return fAltitude; }
    Float_t GetAngle() const { return fAngle; }
    const TArrayF& GetWaveLength() const { return fWaveLength; }
    const TArrayF& GetTransmission() const { return fTransmission; }
    const TArrayF& GetOzone() const { return fOzone; }
    const TArrayF& GetRayl() const { return fRayl; }
    const TArrayF& GetAerosol() const { return fAerosol; }

       void GetWaveLength(Float_t *array, Int_t n) const;
       void GetTransmission(Float_t *array, Int_t n) const;
       void GetOzone(Float_t *array, Int_t n) const;
       void GetRayl(Float_t *array, Int_t n) const;
       void GetAerosol(Float_t *array, Int_t n) const;

       void GetTransferOutput( LowtranTransferOutput& ) const;

       const char* GetTape5() const { return fTape5.Data(); }
       const char* GetTape6() const { return fTape6.Data(); }
       const char* GetTape7() const { return fTape7.Data(); }
       const char* GetTape8() const { return fTape8.Data(); }

       void SetTape5( const char* file ) { fTape5 = file; }
       void SetTape6( const char* file ) { fTape6 = file; }
       void SetTape7( const char* file ) { fTape7 = file; }
       void SetTape8( const char* file ) { fTape8 = file; }

    void SyncTapeNames();
    static LowtranManager* Get();
    
private:
    LowtranManager();

    void LoadLwtrnOutput();

    static LowtranManager* fgMe;

    TString fTape5;
    TString fTape6;
    TString fTape7;
    TString fTape8;

    Float_t fR;
    Float_t fAltitude;
    Float_t fAngle;

    TArrayF fWaveLength;
    TArrayF fTransmission;
    TArrayF fOzone;
    TArrayF fRayl;
    TArrayF fAerosol;

    EsafConfigClass(Atmosphere,LowtranManager)

    ClassDef(LowtranManager,0)
};

#endif  /* __LOWTRANMANAGER_HH__ */

