// ESAF : Euso Simulation and Analysis Framework
// $Id: EarthVector.hh,v 1.8 2004/11/02 15:36:17 moreggia Exp $
// Sylvain Moreggia created Mar, 15 2004
// Vector utility for simulation in the atmosphere


#ifndef __EARTHVECTOR_HH_
#define __EARTHVECTOR_HH_

#include "euso.hh"
#include "TVector3.h"
#include <iostream>

class EarthVector : public TVector3 {
public:
    // ctor
    EarthVector(Double_t x = 0.0, Double_t y = 0.0, Double_t z = 0.0) : TVector3(x,y,z) {}

    // ctor from a table of double
    EarthVector(const Double_t* d) : TVector3(d) {}
    
    // ctor from a table of floats
    EarthVector(const Float_t* f) : TVector3(f) {}
    
    // ctor
    EarthVector(const TVector3& v) : TVector3 (v) {}
    
    // cpy ctor
    EarthVector(const EarthVector& e) : TVector3(e) {}
     
    // dtor
    virtual ~EarthVector() {}
    
    // operator '<<' returns (x,y,z)
    friend ostream& operator<<(ostream &os, const EarthVector &v);
    
    // Vertical altitude
    Double_t Zv() const;

    // Test vertical altitude
    Bool_t IsUnderSeaLevel() const;
    
    // Longitude angle (zero taken at MES origin)  //TOFIX: x ->east, y -> north.. to check after MES final decision
    Double_t LocalLongitude() const;
    
    // Latitude angle (zero taken at MES origin)
    Double_t LocalLatitude() const;
    
private:

    ClassDef(EarthVector,0)
};

#endif  /* __EARTHVECTOR_HH_ */

