// $Id: EsafSys.cc,v 1.8 2006/03/17 14:21:38 thea Exp $
// Author: Alessandro Thea   2005/05/19

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EsafSys                                                              *
 *  Package: Base                                                            *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// EsafSys
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "EsafSys.hh"
#include <TSystem.h>

EsafSys* gEsafSys(NULL);


#define str(s) _QUOTE_(s)

#if !defined(ESAFARCH)
#warning ESAFARCH missing, assuming ESAFARCH=i686
#define ESAFARCH i686
#endif

ClassImp(EsafSys)

//_____________________________________________________________________________
EsafSys::EsafSys( Bool_t absolute ) {
    //
    // Constructor
    //

    
    fArch = str(ESAFARCH);
    fRootVers = ROOT_RELEASE;
    fCxxVers = str(ESAFCXX);
    
    if ( absolute && gSystem->Getenv("ESAFSYS") ) {
        fPath = gSystem->Getenv("ESAFSYS");
        gSystem->ExpandPathName(fPath);
    } else 
        fPath       = ".";

    if ( fPath[fPath.Length()-1] == '/') 
        fPath.Remove(fPath.Length()-1);

    CheckDirSyntax(fPath);

    SetAuxPath(AbsolutePath("auxilar"));
    SetBinPath(AbsolutePath("bin/"+fArch));
    SetLibPath(AbsolutePath("lib/"+fArch));
    SetOutPath(AbsolutePath("output"));
    SetCfgPath(AbsolutePath("config"));
    SetTmpPath(AbsolutePath("tmp"));
}

//_____________________________________________________________________________
EsafSys::EsafSys( const char* path ) {
    //
    // Constructor
    //
    fArch = str(ESAFARCH);
    fRootVers = ROOT_RELEASE;
    fCxxVers = str(ESAFCXX);
    
    fPath = path;

    CheckDirSyntax(fPath);

    SetAuxPath(AbsolutePath("auxilar"));
    SetBinPath(AbsolutePath("bin/"+fArch));
    SetLibPath(AbsolutePath("lib/"+fArch));
    SetOutPath(AbsolutePath("output"));
    SetCfgPath(AbsolutePath("config"));
    SetTmpPath(AbsolutePath("tmp"));

}
 
//_____________________________________________________________________________
EsafSys::~EsafSys() {
    //
    // Destructor
    //

    if ( gEsafSys == this ) 
        gEsafSys = NULL;
}

//______________________________________________________________________________
const char* EsafSys::AbsolutePath( const char* path ) const {
    static TString str;
    str = path;
    AbsolutePath(str);
    return str.Data();
}

//______________________________________________________________________________
TString& EsafSys::AbsolutePath( TString& path ) const {
    //
    // .    =>   .
    // ..   =>   ..
    // ./   =>   ./
    // ../  =>   ../
    // ~    =>   ~
    // /    =>   /
    //
    // @tmp => fTmpPath
    // ..
    // 
    
    if ( path.IsNull() )
        path = fPath;
    else if ( path[0]=='/' || path[0]=='~' || path[0]=='.')
        return path;
    else if ( path.BeginsWith("@aux" ) && !( path.Length() > 4 && path[4]!='/' ) )
        path.Replace(0,4,fAuxPath);
    else if ( path.BeginsWith("@out" ) && !( path.Length() > 4 && path[4]!='/' ) )
        path.Replace(0,4,fOutPath);
    else if ( path.BeginsWith("@cfg" ) && !( path.Length() > 4 && path[4]!='/' ) )
        path.Replace(0,4,fCfgPath);
    else if ( path.BeginsWith("@tmp" ) && !( path.Length() > 4 && path[4]!='/' ) )
        path.Replace(0,4,fTmpPath);
    else if ( fPath != "." && fPath != "./" ) 
        path.Prepend(fPath);
    return path;
}

//______________________________________________________________________________
TString& EsafSys::CheckDirSyntax( TString& path ) const {

    if ( path[path.Length()-1] != '/')
        path += '/';
        
    return path;
}

