// ESAF : Euso Simulation and Analysis Framework

// Author: Alba Cappa   Jan 30, 2005

#ifndef __ESINGLEPHOTONSPAINTER_HH__
#define __ESINGLEPHOTONSPAINTER_HH__

#include "EAtmosphere.hh"
#include "TPolyMarker3D.h"
#include <TArrayI.h>

#include <map>

/*******************************************************************************
 *
 * ESinglePhotonsPainter: this class plots the last position of the
 * photons in atmosphere
 *
 ******************************************************************************/

class ESinglePhotonsPainter : public TObject {
public:

    ESinglePhotonsPainter( EAtmosphere* );

    virtual ~ESinglePhotonsPainter();

    virtual void Draw( Option_t* ="" );
    virtual void Paint( Option_t* ="" );
    void DefineMScattCut(const TArrayI& array);

    void BuildPoints();
private:

    void Reset();

    EAtmosphere *fAtmosphere;
    
    // for each kind of photon history BEFORE transmission to detector (5)
    // + 1 for all the photons AFTER transmission to detector
    Double_t fXmin[6], fXmax[6], fYmin[6], fYmax[6], fZmin[6], fZmax[6];
    
    Int_t fMarkerStyle;
    Int_t fMarkerSize;

    std::map< Int_t, TPolyMarker3D* > fPoints_fluo;   // fluo photons BEFORE transmission to detector
    std::map< Int_t, TPolyMarker3D* > fPoints_ckov;   // ckov photons BEFORE transmission to detector
    std::map< Int_t, TPolyMarker3D* > fPoints_trans;  // all photons AFTER transmission to detector

    TArrayI fIndex_list;             // if MScatt cut : list of indices of selected photons -- ALL photons otherwise

    ClassDef(ESinglePhotonsPainter,1)

};

#endif  /* __EDETECTORPHOTONPLAYER_HH_ */


