// ESAF : Euso Simulation and Analysis Framework
// $Id: RecoCellHit.hh,v 1.7 2006/03/23 16:17:15 naumov Exp $
// Marco Pallavicini created Nov, 18 2003
// simple macrocell hit
//
#ifndef __RECOCELLHIT_HH_
#define __RECOCELLHIT_HH_

#include "euso.hh"

class RecoCellHit {
public:
	
    // ctor
    RecoCellHit(Int_t,Int_t,Int_t);

    // dtor
    virtual ~RecoCellHit();

    // get pixel id
    inline Int_t GetPixelId() const { return fPixelId; }

    // number of photoelectrons detected
    inline Int_t GetCounts() const { return fCounts; }

    // get gtu
    inline Int_t GetGtu() const { return fGtu; }

    inline Int_t GetCountsSignal()     { return fCountsSignal;             }
    inline Int_t GetCountsFluo()       { return fCountsFluorescent;        }
    inline Int_t GetCountsCherDirect() { return fCountsCherenkovDirect;    }
    inline Int_t GetCountsCherScatt()  { return fCountsCherenkovScattered; }
    inline Int_t GetCountsNoise()      { return fCountsNoise;              }

    inline void SetCounts(Int_t n)           { fCounts = n;                   }
    inline void SetCountsSignal(Int_t n)     { fCountsSignal = n;             }
    inline void SetCountsFluo(Int_t n)       { fCountsFluorescent = n;        }
    inline void SetCountsCherDirect(Int_t n) { fCountsCherenkovDirect = n;    }
    inline void SetCountsCherScatt(Int_t n)  { fCountsCherenkovScattered = n; }
    inline void SetCountsNoise(Int_t n)      { fCountsNoise = n;              }

    inline void AddCountFluo()       { fCountsFluorescent += 1;        }
    inline void AddCountCherDirect() { fCountsCherenkovDirect += 1;    }
    inline void AddCountCherScatt()  { fCountsCherenkovScattered += 1; }
    
private:

    Int_t fPixelId;                  // unique pixel id
    Int_t fCounts;                   // number of detected photo-electrons
    Int_t fGtu;                      // gtu number 

                                     // WARNING: The information below is MC and can not be altered from the real data
    Int_t fCountsSignal;             // number of fluorescent photo-electrons counts
    Int_t fCountsFluorescent;        // number of fluorescent photo-electrons counts
    Int_t fCountsCherenkovDirect;    // number of direct cherenkov photo-electrons counts
    Int_t fCountsCherenkovScattered; // number of direct cherenkov photo-electrons counts
    Int_t fCountsNoise;              // number of noise counts
                                     // WARNING: The information above is MC and can not be altered from the real data

    ClassDef(RecoCellHit,0)
};

#endif  /* __RECOCELLHIT_HH_ */

