// ESAF : Euso Simulation and Analysis Framework
// $Id: EusoCluster.hh,v 1.11 2004/09/14 12:40:21 naumov Exp $
// R. Pesce created Jan, 23 2004

#ifndef __EUSOCLUSTER_HH_
#define __EUSOCLUSTER_HH_

#include <vector>
#include <map>
#include "euso.hh"

class EusoCluster {
public:
    // ctor
    EusoCluster();

    // dtor
    virtual ~EusoCluster();

    // add a point in cluster
    void AddPoint( Int_t id );

    // getters
    inline Int_t GetNumPoints() const { return fNumPoints; }
    inline Double_t GetThetaMin() const { return fThetaMin; }
    inline Double_t GetThetaMax() const { return fThetaMax; }
    inline Double_t GetPhiMin() const { return fPhiMin; }
    inline Double_t GetPhiMax() const { return fPhiMax; }
    inline Int_t GetGtuMin() const { return fGtuMin; }
    inline Int_t GetGtuMax() const { return fGtuMax; }
    inline vector<Int_t> GetPixelVector() { return fPixelId; }
    const Int_t GetPixelId( Int_t i );
    inline Bool_t GetFitted() const { return fIsFitted; }
    inline Double_t GetSlope() const { return fSlope; }
    inline Double_t GetOffset() const { return fOffset; }
    inline Double_t GetAbsoluteDeviation() const { return fAbsoluteDeviation; }
    
    // setters
    inline void SetThetaMin( Double_t th ) { fThetaMin = th; }
    inline void SetThetaMax( Double_t th ) { fThetaMax = th; }
    inline void SetPhiMin( Double_t phi ) { fPhiMin = phi; }
    inline void SetPhiMax( Double_t phi ) { fPhiMax = phi; }
    inline void SetGtuMin( Int_t gtu ) { fGtuMin = gtu; }
    inline void SetGtuMax( Int_t gtu ) { fGtuMax = gtu; }
    inline void SetFitted( Bool_t isfit ) { fIsFitted = isfit; }
    inline void SetSlope( Double_t slope ) { fSlope = slope; }
    inline void SetOffset( Double_t offset ) { fOffset = offset; }
    inline void SetAbsoluteDeviation( Double_t abdev ) { fAbsoluteDeviation = abdev; }

private:

    Int_t fNumPoints;   // number of points in cluster
    Double_t fThetaMin;
    Double_t fThetaMax;
    Double_t fPhiMin;
    Double_t fPhiMax;
    Int_t fGtuMin;
    Int_t fGtuMax;

    Bool_t fIsFitted; // true if cluster if fitted
    // parameters of straight line that fit the points
    Double_t fSlope;
    Double_t fOffset;
    Double_t fAbsoluteDeviation;

    // points of cluster
    vector<Int_t> fPixelId; // not is unique id but reference to fPixelData vector in RecoEvent
    ClassDef(EusoCluster,0)
};

#endif  /* __EUSOCLUSTER_HH_ */

