// ESAF : Euso Simulation and Analysis Framework
// $Id: HmaxByShapeMethod.hh,v 1.2 2006/03/15 10:50:36 naumov Exp $
// Dmitry V. Naumov created Jun,  6 2004 as HmaxByShapeMethodModule and updated 18.02.2006 as HmaxByShapeMethod
// belonging to the energy package. The former becomes obsolete

#ifndef __HMAXBYSHAPEMETHOD_HH_
#define __HMAXBYSHAPEMETHOD_HH_

#include <TString.h>
#include <TVector3.h>
#include "EsafMsgSource.hh"
#include "EnergyModule.hh"

class TF1;
class TH1F;

class HmaxByShapeMethod : public EsafMsgSource, public TObject {
public:
    HmaxByShapeMethod(EnergyModule*);                               // ctor
    virtual ~HmaxByShapeMethod();                                   // dtor

    Bool_t Fit(TH1F*);                                              // main method which makes the fit 
    void Clear();                                                   // memory clean
    void DensityToAltitude();

    void Set(Float_t energy,Float_t timemax,TVector3 showerdir,TVector3 showermaxdir);
    void SetEnergy(Float_t e)            { fEnergy = e;         }   // Set energy
    void SetShowerDir(TVector3 n)        { fShowerDir = n;      }   // Set unit vector along the shower direction
    void SetShowerMaxDir(TVector3 n)     { fShowerMaxDir = n;   }   // Set unit vector pointing to the EUSO from the shower maximum
    void SetFluoMaxTime(Float_t t)       { fFluorMaxTime = t;   }   // Set Time corresponding to fluorescence maximum

    Double_t  GetDensity()               { return fDensity;     }
    Double_t  GetAltitude()              { return fAltitude;    } 
    Double_t  GetSigma()                 { return fSigma;       } 

    EnergyModule* GetEnergyModule()      { return fEnergyModule;}
private:
    EnergyModule *fEnergyModule;               // pointer to the EnergyModule object
    Double_t      fEnergy;                     // assumed energy
    Double_t      fAltitude;                   // reconstructed altitude of shower maximum
    Double_t      fSigma;                      // reconstructed sigma of shower (in microsecond)
    Double_t      fDensity;                    // reconstructed air density at the shower maximum position
    TVector3      fShowerDir;                  // reconstructed unit vector along the shower direction 
    TVector3      fShowerMaxDir;               // reconstructed unit vector pointing to the EUSO from the shower maximum
    TF1*          fFitFcn;                     // Fit function
    Float_t       fChi2;                       // Chi2/NDF after the fit
    Double_t      fFluorMaxTime;               // Time corresponding to fluorescence maximum
    ClassDef(HmaxByShapeMethod,0)
};

#endif  /* __HMAXBYSHAPEMETHOD_HH_ */

