// $Id: RecoShowerTrack.hh,v 1.9 2006/02/28 16:39:05 naumov Exp $
// Author: Dmitry.Naumov   2005/10/11

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoShowerTrack                                                      *
 *  Package: <Energy>                                                        *
 *  Coordinator: <Dmitry.Naumov>                                             *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOSHOWERTRACK_HH__
#define __RECOSHOWERTRACK_HH__

#include "euso.hh"
#include "EsafMsgSource.hh"
#include "RecoShowerStep.hh"

#include <TVector3.h>

#include <vector>
#include <string>
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoShowerTrack                                                            //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////
class SlastShowerGenerator;
class TCanvas;
class FluoCalculator;
class EsafSpectrum;
class SlastShowerGenerator;
class O1_ClearSkyPropagator;
class DetectorGeometry; 
class Ground;
class EnergyModule; 
class TList;

class RecoShowerTrack: public EsafMsgSource {
public:
                         RecoShowerTrack();
                         RecoShowerTrack(Float_t t, Float_t p, Float_t h, Float_t tfov, Float_t pfov);
           void          Constructor();
    virtual             ~RecoShowerTrack();
    inline void          SetThetaPhi(Float_t t, Float_t p);
    inline void          SetHmax(Float_t h);
    inline void          SetThetaPhiFoV(Float_t t, Float_t p);
    inline void          SetEnergyModule(EnergyModule* e) { fEnergyModule = e; }
           void          SetEnergy(Float_t e)             { fEnergy = e;}
           Bool_t        MakeTrack(); 
    inline void          Add(RecoShowerStep); 
           void          Clear(); 
           void          Debug(TCanvas*,Int_t,Option_t *opt="");    
           void          DoDebugHistos();

    const  vector<RecoShowerStep>& GetSteps( ) const { return fSteps; }
    UInt_t const                   GetSize() const  { return fSteps.size();  } 
    inline const RecoShowerStep&   GetStep( UInt_t ) const;
    const  RecoShowerStep&         GetStepThetaPhi(Float_t Theta, Float_t Phi) const;
    inline TVector3                GetShowerDir()   { return fShowerDir;   }
    inline TVector3                GetEusoDir()     { return fEusoDir;     }
    inline Float_t                 GetHmax()        { return fHmax;        }
           TList*                  GetDebugList()   { return fDebugList;   }
           Double_t                GetXmin()        { return fXmin;        }
           Double_t                GetYmin()        { return fYmin;        }
           Double_t                GetZmin()        { return fZmin;        }
           Double_t                GetXmax()        { return fXmax;        }
           Double_t                GetYmax()        { return fYmax;        }
           Double_t                GetZmax()        { return fZmax;        }
           Double_t                GetDepthMax()    { return fDepthMax;    }
private:
    EnergyModule                 *fEnergyModule;                // pointer to energy module object
    Float_t                       fEnergy;
    // Fluorescence and Radiative Transfer
    DetectorGeometry             *fDetectorGeometry;            // detector geometry object
    FluoCalculator               *fFluoCalculator;              // Fluorescent calculator
    O1_ClearSkyPropagator        *fClearSkyPropagator;          // Clear Sky propagator
    EsafSpectrum                 *fEsafSpectrum;                // pointer to EsafSpectrum object
    // Ground 
    Ground                       *fGround;                      // Ground object
    SlastShowerGenerator         *fShowerGenerator;             // ShowerGenerator    
    TVector3                      fShowerDir;                   // unit vector against the shower direction
    TVector3                      fEusoDir;                     // unit vector into the Euso direction
    Float_t                       fHmax;                        // estimate of the shower altitude
    Float_t                       fEusoAlt;                     // Euso Altitude
    string                        fEnergyDistributionName;
    vector <RecoShowerStep>       fSteps;                       // steps of the shower
    TList                        *fDebugList;                   // TList of debug histos
    Double_t                      fXmin;                        // X min of the track
    Double_t                      fYmin;                        // Y min of the track
    Double_t                      fZmin;                        // Z min of the track
    Double_t                      fXmax;                        // X max of the track
    Double_t                      fYmax;                        // Y max of the track
    Double_t                      fZmax;                        // Z max of the track
    Double_t                      fDepthMax;                    // atmosphere depth at the shower maximum
    ClassDef(RecoShowerTrack,0)
};

inline void RecoShowerTrack::SetThetaPhi(Float_t t, Float_t p) {
    fShowerDir.SetMagThetaPhi(1.,t,p);
}

inline void RecoShowerTrack::SetThetaPhiFoV(Float_t t, Float_t p) {
    fEusoDir.SetMagThetaPhi(1.,t,p);
}

inline void RecoShowerTrack::SetHmax(Float_t h) {
    fHmax = h;
}

inline void RecoShowerTrack::Add(RecoShowerStep s) {
    fSteps.push_back(s);
}

inline const RecoShowerStep& RecoShowerTrack::GetStep( UInt_t i ) const {
    // 
    // Return the i-th step of the shower
    // 
    if ( i >= fSteps.size() )
	Error("RecoShowerTrack::GetStep","Index out of range in RecoShowerTrack::GetStep");
    return (fSteps[i]);
}

#endif  /* __RECOSHOWERTRACK_HH__ */

