// $Id: EnergyFitUtils.cc,v 1.23 2006/03/24 08:13:35 naumov Exp $
// Author: Dmitry.Naumov   2005/09/26

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EnergyFitUtils                                                       *
 *  Package: Energy                                                          *
 *  Coordinator: Dmitry.Naumov                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// EnergyFitUtils
//
// 
#include "EnergyFitUtils.hh"
#include "EnergyModule.hh"
#include "EOpticsResponse.hh"
#include "OpticsResponseBuilder.hh"
#include "RecoShowerTrack.hh"
#include "RecoEvent.hh"
#include "RecoPixelData.hh"
#include "ERunParameters.hh"

#include <TMinuit.h>
#include <TF1.h>
#include <TProfile.h>
using namespace TMath;
using namespace sou;

//_____________________________________________________________________________
Double_t Saturation(Int_t pe) {
    //
    //
    //
    Double_t dead_time = EnergyModule::Get()->GetFeDeadTime();
    Double_t gtu       = EnergyModule::Get()->GetRecoEvent()->GetHeader().GetDetector()->GetElectronics()->GetGtuLength();
    return pe*Exp(-pe*dead_time/gtu);
}

//_____________________________________________________________________________
Int_t UnfoldSaturation(Int_t pe) {
    //
    //
    //
    Int_t guess = pe;
    Double_t chi2min=10000;
    for (Int_t i=pe; i<=250; i++) {
           Double_t chi2 = TMath::Abs(Saturation(i)-pe);
           if (chi2 < chi2min) {
               chi2min = chi2;
               guess = i;
           }
    }
    return guess;
}
//_____________________________________________________________________________
Double_t Fit_Energy(Double_t *x, Double_t *par) {

    Float_t time    = x[0];
    Float_t energy  = par[0];
    Float_t tmax    = par[1];
    Float_t sigma   = par[2];

    EnergyModule *fEnergyModule = (EnergyModule*)gMinuit->GetObjectFit();
    if (!fEnergyModule) {
      TF1::RejectPoint();
      return 0;
    }

    Float_t Phi   = fEnergyModule->PhiFoV(time);
    Float_t Theta = fEnergyModule->ThetaFoV(time);
    Float_t Efficacy = fEnergyModule->GetEfficacy(Theta,Phi);

    if (Theta*RadToDeg() > 30.) {
      TF1::RejectPoint();
      return 0;
    }

    if (time >= *(fEnergyModule->GetFCtime()+1) && 
	time <= *(fEnergyModule->GetFCtime()+3)) {
      TF1::RejectPoint();
      return 0;
    }


    GtuPixels_t GtuPixels = fEnergyModule->GetMyGtuPixels();
    Pixels_t OneGtuPixels = GtuPixels[Int_t(time)];
    Pixels_t::iterator itPix;
    Float_t background(0);
    for (itPix = OneGtuPixels.begin(); itPix != OneGtuPixels.end(); itPix++) {
         background += fEnergyModule->BgrPerPixelPerGtu(time);
    }
    Float_t          fQuantumEff = fEnergyModule->GetQuantumEff();
    RecoShowerTrack *fTrack      = fEnergyModule->GetRecoShowerTrack();

    const RecoShowerStep& s = fTrack->GetStepThetaPhi(Theta,Phi);
    Float_t fAttenuation  = s.GetAttenuation();
    Float_t fYield        = s.GetFluoYield();
    Float_t fShowerLength = s.GetLength();
    Float_t fRmax         = (s.GetDir()).Mag();

    // Use Gauss
    Double_t var = Power((time-tmax)/sigma,2);
    Float_t pe = Exp(-0.5*var)*energy*1.e14*fQuantumEff*Efficacy*fAttenuation*fYield*fShowerLength/(1450.*4.e0*Pi()*fRmax*fRmax) +
                 background;
    return pe;
}


//_____________________________________________________________________________
Double_t TimeDistr(Double_t *x, Double_t *par) {

    Float_t time    = x[0];
    Float_t energy  = par[0];
    Float_t tmax    = par[1];
    Float_t sigma   = par[2];

    EnergyModule *fEnergyModule = (EnergyModule*)gMinuit->GetObjectFit();
    if (!fEnergyModule) {
      TF1::RejectPoint();
      return 0;
    }

    Float_t Phi   = fEnergyModule->PhiFoV(time);
    Float_t Theta = fEnergyModule->ThetaFoV(time);

    if (Theta*RadToDeg() > 30.) {
      TF1::RejectPoint();
      return 0;
    }

    Float_t          fQuantumEff = fEnergyModule->GetQuantumEff();
    RecoShowerTrack *fTrack      = fEnergyModule->GetRecoShowerTrack();

    const RecoShowerStep& s = fTrack->GetStepThetaPhi(Theta,Phi);
    Float_t fAttenuation    = s.GetAttenuation();
    Float_t fYield          = s.GetFluoYield();
    Float_t fShowerLength   = s.GetLength();
    Float_t fRmax           = (s.GetDir()).Mag();

    // Use Gauss
    Double_t var = Power((time-tmax)/sigma,2);
    Float_t pe = Exp(-0.5*var)*energy*1.e14*fQuantumEff*fAttenuation*fYield*fShowerLength/(1450.*4.e0*Pi()*fRmax*fRmax);
    return pe;
}

//_____________________________________________________________________________
Double_t Background(Double_t *x, Double_t *par) {
    //
    //
    //

    Float_t time    = x[0];
    EnergyModule *fEnergyModule = (EnergyModule*)gMinuit->GetObjectFit();
    if (!fEnergyModule) {
      TF1::RejectPoint();
      return 0;
    }

    Float_t Theta = fEnergyModule->ThetaFoV(time);

    if (Theta*RadToDeg() > 30.) {
      TF1::RejectPoint();
      return 0;
    }

    GtuPixels_t GtuPixels = fEnergyModule->GetMyGtuPixels();
    Pixels_t OneGtuPixels = GtuPixels[Int_t(time)];
    Pixels_t::iterator itPix;
    Float_t background(0);
    for (itPix = OneGtuPixels.begin(); itPix != OneGtuPixels.end(); itPix++) {
         background += fEnergyModule->BgrPerPixelPerGtu(time);
    }

    return background;
}


