// ESAF : Euso Simulation and Analysis Framework
// $Id: HoughFit.hh,v 1.5 2006/03/20 15:54:11 pesce Exp $
// Elena Taddei created Mar, 29 2005

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: HoughFit                                                             *
 *  Package: Fitting                                                         *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __HOUGHFIT_HH_
#define __HOUGHFIT_HH_

#include <vector>
#include "euso.hh"
#include "EsafConfigurable.hh"
#include "EsafMsgSource.hh"
#include <TObject.h>
#include <TList.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//  HoughFit                                                                //
//                                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////


class HoughFit : public EsafConfigurable, public EsafMsgSource {
public:
    // ctor
    HoughFit( vector<Double_t> x, vector<Double_t> y,  vector<Int_t> c, Int_t fDoSelection, Float_t ex, Float_t ey, vector<Int_t> id = vector<Int_t>() );
	
    // dtor
    virtual ~HoughFit();

    // getters
    inline Double_t GetSlope() const { return fSlopeToPass; }
    inline Double_t GetOffset() const { return fOffsetToPass; }
    inline Double_t Getwidth() const { return fWidth; }
    inline Int_t GetNumPoints() const { return fNumPoints; }
    inline Int_t GetNumPointsSelected() const { return fNumPointsSelected; }
    inline Double_t GetMinX() const { return fMinX; }
    inline Double_t GetMinY() const { return fMinY; }
    inline Double_t GetMaxX() const { return fMaxX; }
    inline Double_t GetMaxY() const { return fMaxY; }
    inline Float_t GetAbsDev() const { return fAbsDev; }
    inline Float_t GetMaxToMedium() const { return fCMaxToCAverage; }
    inline Float_t GetMaxToMediumSel() const { return fCMaxToCAverageSel; }

    inline Bool_t IsSignal() const { return fThereIsSignal; }

    inline vector<Double_t> GetXSel() { return fXSel; }
    inline vector<Double_t> GetYSel() { return fYSel; }
    inline vector<Int_t> GetCSel() { return fCSel; }
    inline vector<Int_t> GetIdSel() { return fIdSel; }

    EsafConfigClass(Reco, HoughFit)

private:
    Float_t fCMaxToCAverage;	
    Float_t fCMaxToCAverageSel;
    Float_t fCMaxToCAverageTmp;
    Float_t fCMaxToCAverageFluttTmp;
    Float_t fFluttBackgHSpace;

    Float_t fWidthFractionSinusoidal;
    Float_t fWidthFractionLinear;
    
    vector<Double_t> fXSel;
    vector<Double_t> fYSel;
    vector<Int_t> fCSel;
    vector<Int_t> fIdSel;
    Bool_t fThereIsSignal;
    Bool_t fDoGraph;

    Float_t fEpsX, fEpsY;
    Float_t fAbsDev;
    Int_t fNumPoints;
    Double_t fMinX,fMinY,fMaxX,fMaxY;
    vector<Double_t> fPointsX;
    vector<Double_t> fPointsY;
    vector<Int_t> fCounts;
    vector<Int_t> fId;
    vector<Double_t> fPointsXprime;
    vector<Double_t> fPointsYprime;
    
    Float_t tmpfMinX, tmpfMinY, tmpfMaxX, tmpfMaxY;
    Float_t fScale;
    Double_t fX0, fY0;

    Double_t fSlopeToPass;
    Double_t fOffsetToPass;
    Double_t fSlope;
    Double_t fOffset;
    Double_t fWidth;
    Int_t fNumPointsSelected;
    
    void DoSinusoidal(Int_t firsttimedo);
    void DoLinear(Int_t firsttimedo);
    void DoLinear2(Int_t firsttimedo);
    void DoNumerical();
    void Clear();
    void PreProcess();

    TList *fListObj;

    ClassDef(HoughFit,0)
};

#endif  /* __HOUGHFIT_HH_ */

