// $Id: RecoTreePainter.cc,v 1.8 2006/03/08 16:07:48 naumov Exp $
// Author: Dmitry.Naumov   2006/02/03

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoTreePainter                                                      *
 *  Package: RecoViewer                                                      *
 *  Coordinator: Dmitry.Naumov                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// RecoTreePainter
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "RecoTreePainter.hh"
#include "RecoRootEvent.hh"
#include "EGViewer.hh"
#include "ESystemOfUnits.hh"

#include <TH1F.h>
#include <TH1D.h>
#include <TH2F.h>
#include <TProfile.h>
#include <TProfile2D.h>
#include <TList.h>
#include <TSystem.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TMath.h>
#include <TF1.h>
#include <TLegend.h>
#include <TLegend.h>
#include <TGraphErrors.h>

ClassImp(RecoTreePainter)

using namespace sou;
//_____________________________________________________________________________
RecoTreePainter::RecoTreePainter(TChain* tree): EsafMsgSource(), fEvent(NULL), fSaveEpsFiles(kFALSE) {
    //
    //
    //
    if (!tree) Error("RecoTreePainter","ctor must be initialized with valid TTree pointer to the recotree\n");
    fHistoList = new TList;
    fEvent = new RecoRootEvent;
    tree->SetBranchAddress("events",&fEvent);
    fNevents = tree->GetEntries();
    fTree = tree;
    Info("RecoTreePainter",Form("Initialized TTree with %d entries",fNevents));

    //
    left_edge["Energy"] = 13;
    left_edge["Theta"]  = 0;

    right_edge["Energy"] = 15;
    right_edge["Theta"]  = 102;
    fFilename = "EnergyPlots";
    gStyle->SetCanvasColor(0);
    fEpsDir = "./";

}

//_____________________________________________________________________________
RecoTreePainter::~RecoTreePainter() {
    //
    // Destructor
    //
}


//_____________________________________________________________________________
void RecoTreePainter::Loop() {
    //
    //
    //
    Int_t   progress    = 0;

    for (Int_t i(0); i<fNevents; i++) {
         fTree->GetEntry(i);
         FillHistos();

         if (100.*i/fNevents >= progress) {
	     Msg(EsafMsg::Info).SetProgress(progress);
	     Msg(EsafMsg::Info) << "Loop():" << MsgCount;
	     progress+=10;
          }
    }
    if (progress <= 100) Msg(EsafMsg::Info) << MsgDispatch;
}

//_____________________________________________________________________________
void RecoTreePainter::FillHistos() {
    //
    //
    //
    map <string, Bool_t>::iterator it;
    MakeBaseDataMap();
    // BULK of simulated information
    TH1* h3 = GetHisto("EnergySimulated_1D","Energy Simulated","1D",100,left_edge["Energy"],right_edge["Energy"],0,0,0);
    h3->SetXTitle("Simulated Energy/EeV");
    h3->SetYTitle("Entries");
    h3->Fill(fSim["EnergySim"]);
    for (it=fCuts.begin(); it != fCuts.end(); it++) {
         TString name  = "EnergySimulated_1D"  + it->first;
         TString title = "EnergySimulated in " + it->first;
         TH1* h = GetHisto(name,title,"1D",100,left_edge["Energy"],right_edge["Energy"],0,0,0);
	 if (it->second) h->Fill(fSim["EnergySim"]);

    }
    h3 = GetHisto("ThetaSimulated_1D","Theta Simulated","1D",100,left_edge["Theta"],right_edge["Theta"],0,0,0);
    h3->SetXTitle("Simulated Theta/deg");
    h3->SetYTitle("Entries");
    h3->Fill(fSim["ThetaSim"]*TMath::RadToDeg());

    if (!Accept()) return;

    TH1* h1 = GetHisto("EnergySim_vs_Rec_2D","Energy Sim vs Rec","2D",100,left_edge["Energy"],right_edge["Energy"],
                                                                   100,left_edge["Energy"],right_edge["Energy"]);
    h1->SetXTitle("Simulated Energy/EeV");
    h1->SetYTitle("Reconstructed Energy/EeV");
    h1->Fill(fSim["EnergySim"],fRec["EnergyRec"]);

    for (it=fCuts.begin(); it != fCuts.end(); it++) {
         TString name  = "EnergySim_vs_Rec"  + it->first;
         TString title = "EnergySim_vs_Rec " + it->first;
         TH1* h = NULL;
	 // Fill Energy Simulated vs Energy Reconstructed
	 h = GetHisto(name+"_2D",title,"2D",100,left_edge["Energy"],right_edge["Energy"],
                                            100,left_edge["Energy"],right_edge["Energy"]);
         if (it->second) h->Fill(fSim["EnergySim"],fRec["EnergyRec"]);
	 // Fill Relative Error of vs Energy Simulated
	 name  = "EnergyErrorRelative_vs_Energy" + it->first;
	 title = "EnergyError Relative vs Energy"+ it->first;
	 h = GetHisto(name+"_2D",title,"2D",100,left_edge["Energy"],right_edge["Energy"],
                                            100,-2,2);
         if (it->second) h->Fill(fSim["EnergySim"],fRec["EnergyErrorRelative"]);
	 // Fill Relative Error of vs signal/noise ratio
	 name  = "EnergyErrorRelative_vs_SignalToNoise" + it->first;
	 title = "EnergyError Relative vs S/N"+ it->first;
	 h = GetHisto(name+"_2D",title,"2D",100,0,100,100,-2,2);
         if (it->second) h->Fill(fRec["SignalToNoise"],fRec["EnergyErrorRelative"]);
	 // Fill signal/noise ratio vs Energy Simulated
	 name  = "SignalToNoise_vs_EnergySim" + it->first;
	 title = "S/N vs Energy"+ it->first;
	 h = GetHisto(name+"_2D",title,"2D",100,left_edge["Energy"],right_edge["Energy"],100,0,100);
         if (it->second) h->Fill(fSim["EnergySim"],fRec["SignalToNoise"]);
	 // Fill simulated Theta angle for this theta bin
	 name  = "SimulatedTheta"+it->first;
	 title = "Simulated Theta in "+it->first;
	 h = GetHisto(name,title,"1D",100,left_edge["Theta"],right_edge["Theta"],0,0,0);
	 if (it->second) h->Fill(fSim["ThetaSim"]*TMath::RadToDeg());
	 // Fill reconstructed Energy  for this theta bin
	 name  = "ReconstructedEnergy:"+it->first;
	 title = "Reconstructed Energy in "+it->first;
	 h = GetHisto(name,title,"1D",100,left_edge["Energy"],right_edge["Energy"],0,0,0);
	 if (it->second) h->Fill(fRec["EnergyRec"]);
	 // Fill simulated Energy  for this theta bin
	 name  = "SimulatedEnergyAccepted:"+it->first;
	 title = "Simulated and accepted Energy in "+it->first;
	 h = GetHisto(name,title,"1D",100,left_edge["Energy"],right_edge["Energy"],0,0,0);
	 if (it->second && TMath::Abs(fRec["EnergyErrorRelative"])<0.3) h->Fill(fSim["EnergySim"]);
	 // Fill statistical expectation for the resolution
	 name  = "StatisticalExpectation"+it->first;
	 title = "Statistical Expectation in "+it->first;
	 h = GetHisto(name,title,"1D",100,0,100,0,0,0);
         if (it->second) h->Fill(fRec["SignalToNoise"],1./TMath::Sqrt(fRec["Signal"]));
   }


    TProfile2D* h2 = (TProfile2D*)GetHisto("EnergyError_prof2","Relative Energy Error","prof2",10,left_edge["Energy"],right_edge["Energy"],
                                                         10,left_edge["Theta"],right_edge["Theta"]);
    h2->SetXTitle("Simulated Energy/EeV");
    h2->SetYTitle("Simulated Theta/deg");
    h2->Fill(fSim["EnergySim"],fSim["ThetaSim"]*TMath::RadToDeg(),fRec["EnergyErrorRelative"]);

    TH2F* h21 = (TH2F*)GetHisto("EnergyRatio_2D","Reconstructed/Simulated","2D",100,left_edge["Energy"],right_edge["Energy"],100,0,3);
    h21->SetXTitle("Simulated Energy/EeV");
    h21->SetYTitle("Reconstructed/Simulated Energy");
    h21->Fill(fSim["EnergySim"],fRec["EnergyRec"]/fSim["EnergySim"]);

    TH1 *h4 = GetHisto("Altitude_vs_Theta","Altitude vs Theta","2D",100,left_edge["Theta"],right_edge["Theta"],100,0,25);
    h4->Fill(fSim["ThetaSim"]*TMath::RadToDeg(),fRec["Altitude"]);
}
//_____________________________________________________________________________
Bool_t RecoTreePainter::Accept() {
     //
     //
     //
     if (fEvent->GetRecoEnergy().GetEnergy() == 5.e14)                                            return kFALSE;
     if (fEvent->GetRecoEnergy().GetQuality() < 0 || fEvent->GetRecoEnergy().GetQuality()> 4)     return kFALSE;
     if (fEvent->GetRecoEnergy().GetEnergy() <= 0 ) return kFALSE;
     return kTRUE;
}

//_____________________________________________________________________________
void RecoTreePainter::MakeBaseDataMap() {
    //
    //
    //
    fSim["EnergySim"] = (Double_t)TMath::Log10(fEvent->GetTruth().GetEnergy());
    fSim["ThetaSim"]  = (Double_t)fEvent->GetTruth().GetTheta();
    fSim["PhiSim"]    = (Double_t)fEvent->GetTruth().GetPhi();

    fRec["EnergyRec"]           = (Double_t)TMath::Log10(fEvent->GetRecoEnergy().GetEnergy());
    fRec["EnergyError"]         = (Double_t)fEvent->GetRecoEnergy().GetEnergyMCError();
    fRec["EnergyErrorRelative"] = (Double_t)fEvent->GetRecoEnergy().GetEnergyMCError()/fEvent->GetTruth().GetEnergy();
    fRec["SignalToNoise"]       = (Double_t)fEvent->GetRecoEnergy().GetSignalCluster()/TMath::Sqrt(fEvent->GetRecoEnergy().GetNoiseCluster());
    fRec["Signal"]              = (Double_t)fEvent->GetRecoEnergy().GetSignalCluster();
    fRec["Altitude"]            = (Double_t)fEvent->GetRecoEnergy().GetAltitude()/km;

    // seting up the cuts
    fCuts["0<Theta<15"]  = 0*TMath::DegToRad()  < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 15*TMath::DegToRad();
    fCuts["15<Theta<30"] = 15*TMath::DegToRad() < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 30*TMath::DegToRad();
    fCuts["30<Theta<45"] = 30*TMath::DegToRad() < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 45*TMath::DegToRad();
    fCuts["45<Theta<60"] = 45*TMath::DegToRad() < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 60*TMath::DegToRad();
    fCuts["60<Theta<75"] = 60*TMath::DegToRad() < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 75*TMath::DegToRad();
    fCuts["75<Theta<102"] = 75*TMath::DegToRad() < fEvent->GetTruth().GetTheta() && fEvent->GetTruth().GetTheta() <= 102*TMath::DegToRad();

}

//_____________________________________________________________________________
void RecoTreePainter::DrawAll() {
    //
    //
    //
    EGViewer *gv = new EGViewer();
    gv->SetName("EnergyReconstuction");
    gv->SetTitle("Energy Reconstuction");
    map <string, Bool_t>::iterator it;
    for (it=fCuts.begin(); it != fCuts.end(); it++) {
         TCanvas *c = gv->FindCanvas((it->first).c_str());
         if(!c )  c = gv->AddTab((it->first).c_str(),(it->first).c_str());
         c->Clear();
         c->cd();
	 c->Divide(3,3);
	 TString name, title;
	 // draw simulated theta distribution: all events and superimposed in the given bin
	 c->cd(1);
         TH1* h1 = GetHisto("ThetaSimulated_1D","Theta Simulated","1D",0,0,0,0,0,0);
	 h1->Draw();
	 name  = "SimulatedTheta"+it->first;
	 TH1* h2 = GetHisto(name,"","1D",0,0,0,0,0,0);
	 h2->Draw("same");
	 h2->SetXTitle("Simulated #{Theta}/deg");
	 h2->SetLineColor(kRed);
	 // draw simulated energy vs reconstructed energy
	 c->cd(2);
         name  = "EnergySim_vs_Rec"  + it->first;
	 TH1* h3 = GetHisto(name+"_2D","","2D",0,0,0,0,0,0);
	 h3->Draw();
	 h3->SetXTitle("Simulated log_{10}Energy");
	 h3->SetYTitle("Reconstructed log_{10}Energy");
	 // draw Relative Error of vs Energy Simulated
	 c->cd(3);
	 name  = "EnergyErrorRelative_vs_Energy" + it->first;
	 TH1* h4 = GetHisto(name+"_2D","","2D",0,0,0,0,0,0);
	 h4->SetXTitle("Simulated log_{10}Energy");
	 h4->SetYTitle("#Delta E/E");
	 h4->Draw();
	 // draw simulated energy superimposed by reconstructed energy
	 c->cd(4);
	 name = "EnergySimulated_1D"  + it->first;
         TH1* h5 = GetHisto(name,"","1D",0,0,0,0,0,0);
         h5->Draw();
	 h5->SetLineWidth(2);
	 h5->SetMinimum(0);
	 name  = "ReconstructedEnergy:"+it->first;
	 TH1* h6 = GetHisto(name,"","1D",0,0,0,0,0,0);
	 h6->SetMarkerStyle(29);
	 h6->SetMarkerSize(0.3);
         h6->Draw("EPsame");
	 h6->SetXTitle("log_{10}Energy");
	 h6->SetLineColor(kRed);
	 h5->Rebin(4);
	 h6->Rebin(4);
	 // draw efficiency as a function of energy
	 c->cd(5);
	 name = "SimulatedEnergyAccepted:"+it->first;
	 TH1* h71 = GetHisto(name,"","1D",0,0,0,0,0,0);
	 h71->Rebin(4);
         h71->Divide(h5);
	 h71->SetMarkerStyle(29);
	 h71->SetMarkerColor(kBlue);
	 h71->Draw("P");
	 h71->SetXTitle("log_{10}Energy");
	 h71->SetLineColor(kBlue);
	 h71->SetMaximum(1.2);
	 // draw Relative Error of vs Signal/Noise ratio
	 c->cd(6);
	 name  = "EnergyErrorRelative_vs_SignalToNoise" + it->first;
	 title = "EnergyError Relative vs S/N"+ it->first;
	 TH2D* h8 = (TH2D*)GetHisto(name+"_2D",title,"2D",0,0,0,0,0,0);
	 h8->Draw();
	 h8->SetXTitle("signal/noise");
	 h8->SetYTitle("#Delta E/E");
	 // draw Signal/Noise ratio as a function of Energy simulated
	 c->cd(7);
	 name  = "SignalToNoise_vs_EnergySim" + it->first;
	 title = "S/N vs Energy"+ it->first;
	 TH2D* h9 = (TH2D*)GetHisto(name+"_2D",title,"2D",0,0,0,0,0,0);
	 h9->Draw();
	 h9->SetXTitle("log_{10}Energy");
	 h9->SetYTitle("signal/noise");
	 // draw sigma and mean of the energy resolution as a function of signal/noise
	 c->cd(8);
	 name  = "EnergyErrorRelative_vs_SignalToNoise" + it->first;
	 TH2D* h10 = (TH2D*)h8->Clone();
	 h10->RebinX(4);
	 h10->FitSlicesY();
	 TH1D* h8_1 = (TH1D*)gDirectory->Get(name+"_2D_1");
	 TH1D* h8_2 = (TH1D*)gDirectory->Get(name+"_2D_2");
         h8_1->SetMarkerColor(kBlue);
	 h8_1->SetMarkerStyle(28);
         h8_1->SetXTitle("signal/noise");
         h8_1->SetYTitle("");
	 h8_1->SetTitle("Energy resolution");
         h8_2->SetMarkerColor(kRed);
	 h8_2->SetMarkerStyle(29);
	 h8_2->SetMarkerSize(0.3);

	 if (h8_1) {
	     h8_1->Draw("PE");
	     h8_1->SetMaximum(2);
	     h8_1->SetMinimum(-1);
	 }
	 if (h8_2) h8_2->Draw("PEsame");
/*	 name  = "StatisticalExpectation"+it->first;
	 TH1* h11 = GetHisto(name,title,"1D",100,0,100,0,0,0);
	 h11->Draw("same");
	 h11->SetLineColor(kBlack);
*/
         // draw sigma and mean of the energy resolution as a function of energy
	 c->cd(9);
	 name  = "EnergyErrorRelative_vs_Energy" + it->first;
	 TH2D* h12 = (TH2D*)GetHisto(name+"_2D",title,"2D",100,left_edge["Energy"],right_edge["Energy"], 100,-2,2);
         TH2D* h13 = (TH2D*)h12->Clone();
	 h13->RebinX(10);
	 h13->FitSlicesY();
	 TH1D* h13_1 = (TH1D*)gDirectory->Get(name+"_2D_1");
	 TH1D* h13_2 = (TH1D*)gDirectory->Get(name+"_2D_2");
	 h13_1->SetMarkerColor(kBlue);
	 h13_1->SetMarkerStyle(28);
         h13_1->SetXTitle("log_{10}Energy");
         h13_1->SetYTitle("");
	 h13_1->SetTitle("Energy resolution");
         h13_2->SetMarkerColor(kRed);
	 h13_2->SetMarkerStyle(29);
	 if (h13_1) {
	     h13_1->Draw("PE");
	     h13_1->SetMaximum(2);
	     h13_1->SetMinimum(-1);
	 }
	 if (h13_2) h13_2->Draw("PEsame");


    }
    TCanvas *c = gv->FindCanvas("c_final");
    if(!c )  c = gv->AddTab("c_final","c_final");
    c->Clear();
    c->cd();
    c->Divide(2,2);
    // draw sigma of energy resolution for all theta angles as a function of energy
    Bool_t first = kTRUE;
    c->cd(1);
    TLegend *legend1 = new TLegend(0.6,0.7,1.0,1.0);
    legend1->SetTextFont(72);
    legend1->SetTextSize(0.02);
    Int_t cut(0);
    for (it=fCuts.begin(); it != fCuts.end(); it++) {
	 TString name  = "EnergyErrorRelative_vs_Energy" + it->first;
	 TH1D* h13_2 = (TH1D*)gDirectory->Get(name+"_2D_2");
	 h13_2->SetMarkerStyle(20+cut);
         h13_2->SetMarkerColor(cut+1);
         cut++;
	 if (first) {
	     h13_2->Draw("P");
             h13_2->SetStats(0);
	     h13_2->SetMinimum(-0.2);
	     h13_2->SetMaximum(3);
             h13_2->SetTitle("#sigma of Energy Resolution");
	     h13_2->SetMarkerSize(0.3);
	     first = kFALSE;
	 }
	 else {
	     h13_2->Draw("Psame");
             h13_2->SetStats(0);
             h13_2->SetTitle("#sigma of Energy Resolution");
	     h13_2->SetMarkerSize(0.3);
	 }
	 legend1->AddEntry(h13_2,TString(it->first).Data(),"p");
    }
    legend1->Draw();
    // draw sigma of energy resolution for all theta angles as a function of signal/noise
    c->cd(2);
    TLegend *legend2 = new TLegend(0.6,0.7,1.0,1.0);
    legend1->SetTextFont(72);
    legend1->SetTextSize(0.02);
    cut = 0;
    first  = kTRUE;
    for (it=fCuts.begin(); it != fCuts.end(); it++) {
	 TString name  = "EnergyErrorRelative_vs_SignalToNoise" + it->first;
	 TH1D* h13_2 = (TH1D*)gDirectory->Get(name+"_2D_2");
	 h13_2->SetMarkerStyle(20+cut);
         h13_2->SetMarkerColor(cut+1);
         cut++;
	 if (first) {
	     h13_2->Draw("P");
	     h13_2->SetMinimum(-0.2);
	     h13_2->SetMaximum(3);
             h13_2->SetStats(0);
             h13_2->SetTitle("#sigma of Energy Resolution");
	     h13_2->SetMarkerSize(0.3);
	     first = kFALSE;
	 }
	 else {
	     h13_2->Draw("Psame");
             h13_2->SetStats(0);
             h13_2->SetTitle("#sigma of Energy Resolution");
	     h13_2->SetMarkerSize(0.3);
	 }
	 legend2->AddEntry(h13_2,TString(it->first).Data(),"p");
    }
    legend2->Draw();
    // draw reconstructed altitude as a function of Theta
    c->cd(3);
    TH2D* h = (TH2D*)GetHisto("Altitude_vs_Theta","Altitude vs Theta","2D",0,0,0,0,0,0);
    h->SetYTitle("Altitude [km]");
    h->SetXTitle("#Theta [deg]");
    h->Draw();
    // draw efficiencies as a function of Energy for various bins in Theta
    c->cd(4);
    TLegend *legend3 = new TLegend(0.6,0.7,1.0,1.0);
    legend3->SetTextFont(72);
    legend3->SetTextSize(0.02);
    cut = 0;
    first  = kTRUE;
    for (it=fCuts.begin(); it != fCuts.end(); it++) {
	 TString name = "SimulatedEnergyAccepted:"+it->first;
	 TH1* h = GetHisto(name,"","1D",0,0,0,0,0,0);
	 h->SetMarkerStyle(20+cut);
         h->SetMarkerColor(cut+1);
         cut++;
	 if (first) {
	     h->Draw("P");
	     h->SetMinimum(-0.2);
	     h->SetMaximum(1.5);
             h->SetStats(0);
             h->SetTitle("Efficiency");
	     first = kFALSE;
	 }
	 else {
	     h->Draw("Psame");
             h->SetStats(0);
             h->SetTitle("#sigma of Energy Resolution");
	 }
	 legend3->AddEntry(h,TString(it->first).Data(),"p");
    }
    legend3->Draw();

    gv->SaveAsPS("EnergyPlots.ps");
}


//_____________________________________________________________________________
TH1* RecoTreePainter::GetHisto(const char *name, const char *title, Option_t *opt, Int_t xbins, Float_t xmin, Float_t xmax,
                                                                                   Int_t ybins, Float_t ymin, Float_t ymax) {
    //
    // 
    //
    TH1* th = (TH1*)fHistoList->FindObject(name);
    if ( !title ) title = name;

    if (!th) {
        if      (TString(opt) == "1D")    th = new TH1F( name, title, xbins, xmin, xmax);
        else if (TString(opt) == "2D")    th = new TH2F( name, title, xbins, xmin, xmax, ybins, ymin, ymax);
        else if (TString(opt) == "prof")  th = new TProfile( name, title, xbins, xmin, xmax);
        else if (TString(opt) == "prof2") th = new TProfile2D( name, title, xbins, xmin, xmax,ybins, ymin, ymax,-2,2);
        else                              Error("GetHisto",Form("Option %s is not supported\n",opt));
        fHistoList->Add(th);
    }

    return th;
}
