// $Id: RecoEnergy.cc,v 1.4 2006/02/28 16:43:21 naumov Exp $
// Author: R.Pesce   2005/01/04

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoEnergy                                                           *
 *  Package: RecoRootEvent                                                   *
 *  Coordinator: Dmitry.Naumov                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// RecoEnergy
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "RecoEnergy.hh"
#include "Etypes.hh"

ClassImp(RecoEnergy)

//_____________________________________________________________________________
RecoEnergy::RecoEnergy() {
    //
    // Constructor
    //
    Clear();
}

//_____________________________________________________________________________
RecoEnergy::RecoEnergy( const RecoEnergy& other ) {
    //
    // Copy Constructor
    //
    other.Copy( *this );
}

//_____________________________________________________________________________
RecoEnergy::~RecoEnergy() {
    //
    // Destructor
    //
    Clear();
}

//_____________________________________________________________________________
void RecoEnergy::Clear() {
    //
    // Clear method
    //
    fQuality        = -kHuge;
    fEnergy         = -kHuge;
    fTimeSigma      = -kHuge;
    fEnergyMCError  = -kHuge;
    fThetaFoVMax    = -kHuge;
    fPhiFoVMax      = -kHuge;
    fAltitudeMax    = -kHuge;
    fDepthMax       = -kHuge;
    fCherenkovFound = kFALSE;
    fTimeCherenkov  = -kHuge;
    fTimeDelay      = -kHuge;
    fTimeFluo       = -kHuge;
    fNoiseCluster   = -kHuge;
    fSignalCluster  = -kHuge;
}

//_____________________________________________________________________________
void RecoEnergy::Copy( TObject& other) const {
    //
    // Copy to a new object
    //
    TObject::Copy( other );
    
    ((RecoEnergy&)other).fQuality       = fQuality; 
    ((RecoEnergy&)other).fEnergy        = fEnergy; 
    ((RecoEnergy&)other).fTimeSigma     = fTimeSigma; 
    ((RecoEnergy&)other).fEnergyMCError = fEnergyMCError; 
    ((RecoEnergy&)other).fThetaFoVMax   = fThetaFoVMax; 
    ((RecoEnergy&)other).fPhiFoVMax     = fPhiFoVMax; 
    ((RecoEnergy&)other).fAltitudeMax   = fAltitudeMax;
    ((RecoEnergy&)other).fDepthMax      = fDepthMax;
    ((RecoEnergy&)other).fCherenkovFound= fCherenkovFound;
    ((RecoEnergy&)other).fTimeCherenkov = fTimeCherenkov;
    ((RecoEnergy&)other).fTimeDelay     = fTimeDelay;
    ((RecoEnergy&)other).fTimeFluo      = fTimeFluo;
    ((RecoEnergy&)other).fNoiseCluster  = fNoiseCluster;
    ((RecoEnergy&)other).fSignalCluster = fSignalCluster;
    ((RecoEnergy&)other).fNoisePerPixel = fNoisePerPixel;
    ((RecoEnergy&)other).fSignalPerPixel= fSignalPerPixel;
}
//_____________________________________________________________________________
void RecoEnergy::SetTimeDelay()  {
    //
    // Compute time difference between time of cherenkov and time of fluo maximum. Valid if cherenkov is found.
    //
    if (fCherenkovFound) fTimeDelay = fTimeCherenkov - fTimeFluo;
}

