// ESAF : Euso Simulation and Analysis Framework
// $Id: ChipTrackingTrgEngine.hh,v 1.14 2005/11/18 13:59:10 pesce Exp $
// M. Pallavicini created Oct, 24 2003

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EusoElectronics                                                      *
 *  Package: Electronics                                                     *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/


#ifndef __CHIPTRACKINGTRGENGINE_HH_
#define __CHIPTRACKINGTRGENGINE_HH_

#include "euso.hh"
#include "TriggerEngine.hh"
#include "ChipTrackSegment.hh"

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  ChipTrackingTrgEngine                                                    //
//                                                                           //
// Tracking trigger built in front end chip logic.                           //
//                                                                           //
// ALGORYTHM:                                                                //
// each chip is capable of finding a track segment of consequtive and        // 
// adjacent pixels.                                                          //
// If the length of a given segment exceeds a threshold, trigger occurs      //
// If not, the macrocell checks if there are 2 segments in nearby front end  //
// chips that summed together exceed the threshold: if this happens, trigger //
// occurs.                                                                   //
//                                                                           //
/////////////////////////////////////////////////////////////////////////////// 


class ChipTrackingTrgEngine : public TriggerEngine {
public:
    // ctor
    ChipTrackingTrgEngine();
    
    // dtor
    virtual ~ChipTrackingTrgEngine();

    // simulate trigger
    virtual void Simulate( MacroCellData* );

    EsafConfigClass(Electronics,ChipTrackingTrgEngine);


private:
    // trigger parameters
    Int_t fThreshold;                 //threshold on pixel hits in a gtu
    Bool_t fRelativeThreshold;        // true if the threshold is relative to the mean value
                                      // of the background in each chip: thresh = back + n*sqrt(back)
   
    Int_t fMinTrackLength;            //algorithm searches for tracks of length 
    Int_t fMaxTrackLength;            //between fMinTrackLength and fMaxTrackLength
    
    Int_t fMinTriggerTrackLength;     //min trigger track length in a single chip
    Int_t fMinTriggerTwoLength;       //min trigger track length in two nearby chips
    Int_t fMaxTwoLength;              // max length for trigger in two nearby chips
    
    Bool_t fAcceptHole;               //true if we accept an hole in a track in a single chip
    Bool_t fOnlyWithSignal;           //if true the algorithm processes only chip with signal

    // clear maps
    void Clear();

    // fill run parameters
    void FillRunPars();

    // fill rootfile event
    void FillEEvent( Int_t );

    // dump the number of tracks for each chip (debug method)
    void Dump( Int_t );

    map<Int_t, vector<ChipTrackSegment> > fTrackSegments;    //map for storing data
    
    ClassDef(ChipTrackingTrgEngine,0)
};

#endif  /* __CHIPTRACKINGTRGENGINE_HH_ */

