// ESAF : Euso Simulation and Analysis Framework
// $Id: FocalSurfaceFileParser.hh,v 1.6 2006/03/01 20:54:01 thea Exp $
// M. Pallavicini - created

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: FocalSurfaceFileParser                                               *
 *  Package: FocalSurfaceFileParser                                          *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/

#ifndef __FOCALSURFACEFILEPARSER_HH__
#define __FOCALSURFACEFILEPARSER_HH__

#include "euso.hh"
#include "EVector.hh"
#include "NumbersFileParser.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// FocalSurfaceFileParser                                                     //
//                                                                            //
// This class inherits from NumberFileParser (config package)                 //
// It is basically an helper class to build the Focal Surface. It is used     //
// by EusoElectronics                                                         //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class FocalSurfaceFileParser : public NumbersFileParser {
public:
    // ctor
    FocalSurfaceFileParser( string& fname );

    // dtor 
    virtual ~FocalSurfaceFileParser();

    // returns PMT number of the current line
    // at each call move one line down
    // returns 0 when number table is over
    Int_t NewPmtId();

    // return macrocell id if we have a new one, otherwise 0
    Int_t NewMacroCell();

    // return elementary cell id if we have a new one, otherwise 0
    Int_t NewElementaryCell();
    
    // return front end chip id if we have a new one, otherwise 0
    Int_t NewFrontEndChip();

    // row and column position in macrocell
    inline Int_t GetRowOffset() { return (Int_t) GetNum( fCurrent, CellRowCol ); }
    inline Int_t GetColOffset() { return (Int_t) GetNum( fCurrent, CellColCol ); }

    // geometry parameters 
    const EVector& Pos(); 
    const EVector& Norm(); 
    const EVector& Dir();

private:
    // current line
    Int_t fCurrent;

    // keep old values 
    Int_t fOldPmt;
    Int_t fOldMC; 
    Int_t fOldChip;
    Int_t fOldEC;

    // geometry vectors
    EVector fPos; 
    EVector fNorm; 
    EVector fDir;
   
    // file format table
    enum FileFormatTable { PmtIdCol=0,       // pmt identifier
                           MCIdCol,          // macrocell identifier
                           PDMIdCol,         // pdm identifier
                           ECIdCol,          // elementary cell identifier
                           ChipIdCol,        // front end chip identifier
                           CellRowCol,       // offset row in macrocell
                           CellColCol,       // offset column in macrocell,
                           PmtTypeCol,       // type pf PMT
                           PosXCol,          // position X
                           PosYCol,          // position Y,
                           PosZCol,          // position Z,
                           NormXCol,         // normal X
                           NormYCol,         // normal Y
                           NormZCol,         // normal Z
                           DirXCol,          // direction X
                           DirYCol,          // direction Y
                           DirZCol           // direction Z
                         };

    ClassDef(FocalSurfaceFileParser,0)
};

#endif  /* __FOCALSURFACEFILEPARSER_HH__ */
