// ESAF : Euso Simulation and Analysis Framework
// $Id: SignalChipTrackingTrgEngine.hh,v 1.4 2005/11/18 13:59:10 pesce Exp $
// R. Pesce created

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SignalChipTrackingTrgEngine                                          *
 *  Package: Electronics                                                     *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/


#ifndef __SIGNALCHIPTRACKINGTRGENGINE_HH_
#define __SIGNALCHIPTRACKINGTRGENGINE_HH_

#include "euso.hh"
#include "TriggerEngine.hh"
#include "ChipTrackSegment.hh"

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  SignalChipTrackingTrgEngine                                              //
//                                                                           //
// Tracking trigger built in front end chip logic.                           //
// This trigger engine works only on the pure signal                         //
//                                                                           //
// ALGORYTHM:                                                                //
// each chip is capable of finding a track segment of consequtive and        // 
// adjacent pixels.                                                          //
// If the length of a given segment exceeds a threshold, trigger occurs      //
// If not, the macrocell checks if there are 2 segments in nearby front end  //
// chips that summed together exceed the threshold: if this happens, trigger //
// occurs.                                                                   //
//                                                                           //
// Special multiple configuration is read from config file                   //
// A specific trigger word is filled respect to the length of recognized     //
// tracks.                                                                   //
//                                                                           //
/////////////////////////////////////////////////////////////////////////////// 


class SignalChipTrackingTrgEngine : public TriggerEngine {
public:
    // ctor
    SignalChipTrackingTrgEngine();
    
    // dtor
    virtual ~SignalChipTrackingTrgEngine();

    // simulate trigger
    virtual void Simulate( MacroCellData* );

    EsafConfigClass(Electronics,SignalChipTrackingTrgEngine);


private:
    ClassDef(SignalChipTrackingTrgEngine,0)

    // trigger parameters
    Bool_t fRelativeThreshold;        // true if the threshold is relative to the mean value
                                      // of the background in each chip: thresh = back + n*sqrt(back)
    Int_t fThreshold;                 // threshold on pixel hits in a gtu
    Int_t fMinTrackLength;            // algorithm searches for tracks of length between
    Int_t fMaxTrackLength;            // fMinTrackLength and fMaxTriggerTrackLength
    
    Int_t fMinTriggerTrackLength;     // min trigger track length in a single chip
    Int_t fMinTriggerTwoLength;       // min trigger track length in two nearby chips
    Int_t fMaxTwoLength;              // max length for trigger in two nearby chips

    Bool_t fAcceptHole;               // true if we accept an hole in a track in a single chip
    Bool_t fOnlyWithSignal;           // if true processes only chip with signal

    // counter for trigger pattern type to insert in the triggerword
    Int_t fPatternCounter;

    // clear maps
    void Clear();

    // fill root event
    void FillEEvent( Int_t );

    // fill run parameters
    void FillRunPars();

    // dump the number of tracks for each chip
    void Dump( Int_t );

    map<Int_t, vector<ChipTrackSegment> > fTrackSegments;    //map for storing data
};

#endif  /* __SIGNALCHIPTRACKINGTRGENGINE_HH_ */

