// $Id: OpticsFactory.hh,v 1.17 2006/03/20 23:41:19 thea Exp $
// Author: M.Pallavicini, D.Demarco

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: OpticsFactory                                                        *
 *  Package: Optics                                                          *
 *  Coordinator: Alessandro.Thea                                             *
 *                                                                           *
 *****************************************************************************/

#ifndef __OPTICSFACTORY_HH__
#define __OPTICSFACTORY_HH__

#include "OpticalSystem.hh"
#include "FocalPlane.hh"
#include "IdealFocalSurface.hh"
#include "WallInteraction.hh"
#include "Baffle.hh"
#include "PmtGeometry.hh"
#include "OpticalAdaptor.hh"
#include "ConfigFileParser.hh"
#include "Config.hh"

class DetectorTransportManager;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// OpticsFactory                                                              //
//                                                                            //
// Singleton & FactoryMethod                                                  //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class OpticsFactory : public EsafConfigurable {
public:
    EsafConfigClass(Optics,OpticsFactory)
private:
    static OpticsFactory *fgMe;
    OpticsFactory();

//    OpticalSystem     *fOptics;
//    FocalPlane        *fFocalPlane;
//    IdealFocalSurface *fIdealFocalSurface;
//    WallInteraction   *fWalls;
//    Baffle            *fBaffle;
//
//    DetectorTransportManager *fTransportManager;

public:
    virtual ~OpticsFactory() {}
    static OpticsFactory *Get();

    void Build();

    virtual     OpticalSystem *MakeOpticalSystem();
    virtual        FocalPlane *MakeFocalPlane();
    virtual IdealFocalSurface *MakeIdealFocalSurface();
    virtual   WallInteraction *MakeWalls();
    virtual            Baffle *MakeBaffle();
    virtual    OpticalAdaptor *MakeOA();
    virtual DetectorTransportManager *MakeTransportManager();

    ClassDef(OpticsFactory,0)
};

#endif /* __OPTICSFACTORY_HH__ */
