// $Id: PureMCRadiativeTransfer.cc,v 1.3 2006/03/20 21:51:35 thea Exp $
// Author: Sylvain Moreggia   2006/03/09

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: PureMCRadiativeTransfer                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// PureMCRadiativeTransfer
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "PureMCRadiativeTransfer.hh"
#include "Atmosphere.hh"  
#include "RadiativeFactory.hh"
#include "ListPhotonsInAtmosphere.hh"
#include "ListPhotonsOnPupil.hh"
#include "EsafRandom.hh"
#include <TROOT.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TProfile.h>
#include <math.h>  
#include "ParentPhoton.hh" 
#include "SinglePhoton.hh" 
#include "Config.hh"
#include "LowtranRadiativeProcessesCalculator.hh"
#include "FastRadiativeProcessesCalculator.hh"
#include "EConst.hh"
#include "BunchOfPhotons.hh"
#include "Ground.hh"

#include "EEvent.hh"
#include "EAtmosphere.hh"
#include "EAtmosphereBunchAdder.hh"
#include "EAtmosphereSingleAdder.hh"

#include "TBenchmark.h"

using namespace TMath;
using namespace sou;
using namespace EConst;


ClassImp(PureMCRadiativeTransfer)

//_____________________________________________________________________________
PureMCRadiativeTransfer::PureMCRadiativeTransfer() : RadiativeTransfer(), fTotalList(0), fPropagator(0),
                                             fNbBunch(0), fNbTot(0), fNbSingles(0) {
    //
    // Constructor
    //

    Msg(EsafMsg::Info) << "Enabled" << MsgDispatch;
    fGround = RadiativeFactory::Get()->GetGround();
    if(!fGround) Msg(EsafMsg::Panic) << "Pb of memory allocation for fGround" << MsgDispatch;
    
    Double_t maxtof = Conf()->GetNum("PureMCRadiativeTransfer.fTofCut")*microsecond;
    fPropagator = new SinglePhotonPropagator(fGround,maxtof);
    
    fMaxPhNb = Conf()->GetNum("PureMCRadiativeTransfer.fMaxPhNb");    
}

//_____________________________________________________________________________
PureMCRadiativeTransfer::~PureMCRadiativeTransfer() {
    //
    // Destructor
    //
    SafeDelete(fPropagator);
}

//______________________________________________________________________________
Bool_t PureMCRadiativeTransfer::ClearMemory() {
    // 
    // Release all the mem hold in the buffers
    //

    Reset();
    vector<SinglePhoton*> emptySingle;
    emptySingle.swap(fTempList);
    
    return kTRUE;
}


//_______________________________________________________________________________________________________________________
PhotonsOnPupil* PureMCRadiativeTransfer::Get(PhotonsInAtmosphere* photons, const DetectorGeometry* dg) {
    //
    // Transport photons in atmosphere : PhotonsOnPupil generated MUST be considered as fake photons
    //



    ////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////// Initializations /////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////


// 1. Preprocesses
    // copy DetectorGeometry
    CopyDetectorGeometry(dg);
        
    // if given list of photons is empty  
    if(!photons) return NULL;

    if(photons->GetType() == "empty") Msg(EsafMsg::Panic) <<"When NoLightSource used, NoRadiativeTransfer should be used"<< MsgDispatch;
    if(photons->GetType() != "list") Msg(EsafMsg::Panic) <<"Wrong PhotonsInAtmosphere format. ListPhotonsInAtmosphere expected."<< MsgDispatch;
    fTotalList = (ListPhotonsInAtmosphere*) photons;



// 2. Get some infos about created light
    // look at number of bunches and photons
    fNbBunch = fTotalList->GetListOfBunch().size();
    if(fNbBunch != 1) Msg(EsafMsg::Panic) << "PureMCRadiativeTransfer code CANNOT handle more than one bunch of photons"<< MsgDispatch;
    fNbTot = 0;
    fNbSingles = 0;
    Double_t nf(0), nc(0);
    for(size_t i=0; i<fNbBunch; i++) {
        fNbTot += (fTotalList->GetListOfBunch()[i])->GetWeight();
        if(fTotalList->GetListOfBunch()[i]->GetType() == Fluo)
	     nf += fTotalList->GetListOfBunch()[i]->GetWeight();
        else nc += fTotalList->GetListOfBunch()[i]->GetWeight();
    }

// some prints
#ifdef DEBUG
    Msg(EsafMsg::Debug) << "SIZE OF LIST OF BUNCHES = "<<fNbBunch << MsgDispatch;
    Msg(EsafMsg::Debug) << "Nb fluo = " <<nf << MsgDispatch;
    Msg(EsafMsg::Debug) << "Nb ckov = " <<nc << MsgDispatch;

    if ( fNbBunch >=1 ) {
        EarthVector track = fTotalList->GetListOfBunch()[fNbBunch - 1]->GetPos() 
                          - fTotalList->GetListOfBunch()[0]->GetPos();
        Msg(EsafMsg::Debug) << "size of the photon track = "<<track.Mag()/km <<" km"<< MsgDispatch;
    }
#endif
    Msg(EsafMsg::Info) << "TOTAL number of photons ="<<fNbTot << MsgDispatch;
    
    
    

// 3. root output init
    EEvent* ev = EEvent::GetCurrent();
    if(ev->GetAtmosphere()) ev->GetAtmosphere()->SetMaxScatOrder(0);

    // if some SinglePhotons created directly by LightSource module, they are saved into root
    if ( ev ) {
        const vector<SinglePhoton*>& list_single_2 = fTotalList->GetListOfSingle();
        size_t nbnotsaved = fTotalList->GetNbNotSaved();
        for (size_t i = list_single_2.size() - nbnotsaved; i<list_single_2.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single_2[i],true,false,0 );
            ev->Fill(sa);
            fTotalList->OneSingleSaved();
        }
    }




    ////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////// ALGO STARTS HERE /////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////
    BunchOfPhotons* bunch = fTotalList->GetBunch(0);
    Double_t remainingPhotons = -HUGE;
    Int_t split_level = Int_t(ceil(fNbTot / fMaxPhNb));
    Msg(EsafMsg::Warning) << "NB of photons to propagate after reduction "<<fNbTot  << MsgDispatch;
    Msg(EsafMsg::Warning) << "Each scattering order simu will be split in "<<split_level<<" parts" << MsgDispatch;
    
    // saves bunch parameters at creation
    if ( ev ) {
        EAtmosphereBunchAdder ba( bunch, true );
        ev->Fill(ba);
    }
    

    // 1. SCATTERING SIMULATION
#ifdef DEBUG
    TString jobMCRT = "Time spent for Scattering simu:";
    TBenchmark gB;
    gB.Start(jobMCRT);
#endif
    fTotalList->ResetBunchCounter();  // for below extraction from list
    for(Int_t level=0; level < split_level; level++) {
        Msg(EsafMsg::Info) << "************ Create single photons, Level : "<<level+1<<" ************"<< MsgDispatch;

        // 1.1 generate single photons
        remainingPhotons = ConvertIntoSingles(*bunch,remainingPhotons);

        Msg(EsafMsg::Info) << "NB of SinglePhoton to PROPAGATE = "<<fNbSingles<< MsgDispatch;
        fNbSingles = 0;


        // 1.2 scattering simu for single photons in fTempList
        fPropagator->GoForPureMC(fTempList);

	// 1.3 Scan fTempList before filling fTotalList, applying relevant cuts :
	//     - if photon status > 4, it is lost
	ScanTempList();

        // 2.4 Move photons from fTempList to fTotalList : fTempList does not contain photons anymore
        //     Only photons which have successfully scattered are written in fTotalList, others are lost
        fTotalList->Add(fTempList);

#ifdef DEBUG
        gB.Stop(jobMCRT);
        MsgForm(EsafMsg::Info,"Time spent for this level :  REAL=%6.2f s   CPU=%6.2f s",gB.GetRealTime(jobMCRT),gB.GetCpuTime(jobMCRT));
#endif
    }


    // 2. saves single photon parameters after scattering simulation
    if ( ev ) {
        const vector<SinglePhoton*>& list_single = fTotalList->GetListOfSingle();
        size_t nbnotsaved = fTotalList->GetNbNotSaved();
        for (size_t i = list_single.size() - nbnotsaved; i<list_single.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single[i],true,true );
            ev->Fill(sa);
            fTotalList->OneSingleSaved();
        }
    }

#ifdef DEBUG
    Msg(EsafMsg::Debug) <<"final list of single, size = " <<fTotalList->GetListOfSingle().size() <<MsgDispatch;
#endif
    if(fTotalList->GetNbNotSaved()) Msg(EsafMsg::Warning) <<fTotalList->GetNbNotSaved()<<" SinglePhoton not saved into root" <<MsgDispatch;


    
    // saves single photons after propagation until detector
    if ( ev ) {
        const vector<SinglePhoton*>& list_single_3 = fTotalList->GetListOfSingle();
        for ( size_t i=0; i<list_single_3.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single_3[i],false,false,i );
            ev->Fill(sa);
        }
    }
    

    //FIXME: small mem leak here.
    return new ListPhotonsOnPupil;
}

//_______________________________________________________________________________________________________________________
Double_t PureMCRadiativeTransfer::ConvertIntoSingles(const BunchOfPhotons& b, Double_t remainingPhotons) {
    //
    // Photon features are set here (lambda, position, direction)
    // SinglePhoton are stored into fTempList for scattering simu
    //
    // if remainingPhotons > -HUGE, means the nb of photons to create is specified
    //

    Int_t nb = 0;
    Double_t rtn = -HUGE;
    Double_t wl, date, tof;
    EarthVector showerpos, dir, diff;
    SinglePhoton* s = 0;
    UInt_t bid = b.GetId();
    PhotonType type = b.GetType();
    tof = 0;
    
    if(remainingPhotons > -HUGE) nb = Int_t(remainingPhotons);
    else nb = Int_t(b.GetWeight());
    
    for(Int_t i=0; i<nb; i++) {
        // get features at creation
        showerpos = b.RandomPosInShower();
        if(fGround->IsUnderGround(showerpos)) continue;
        diff = showerpos - b.GetShowerPos();
        date = b.GetDate() + diff.Dot(b.GetDir().Unit())/Clight();
        dir  = b.RandomDirection();
        wl   = b.GetWlSpectrum().GetLambda();
        s    = new SinglePhoton(type,date,tof,wl,showerpos,showerpos,dir,None,bid);
        fNbSingles++;
        fTempList.push_back(s);
	if(fNbSingles >= fMaxPhNb) {
	    rtn = nb - 1 - i;
	    break;
	}
    }
    
    return rtn;
}

//_______________________________________________________________________________________________________________________
void PureMCRadiativeTransfer::Reset() {
    //
    // get ready for next event
    //
    if(fGround) fGround->Reset();
    if(fPropagator) fPropagator->Reset();
    for(size_t i=0; i<fTempList.size(); i++) {
        SafeDelete(fTempList[i]);
        Msg(EsafMsg::Panic) <<"fTempList SHOULD BE EMPTY, remains : "<<fTempList.size()<<" photons inside"<<MsgDispatch;
	
    }
    fTempList.clear();
    fNbBunch = 0;
    fNbTot = 0;
    fNbSingles = 0;
}

//_______________________________________________________________________________________________________________________
void PureMCRadiativeTransfer::ScanTempList() {
    //
    // remove unrelevant photons (status == LostByCut)
    // if fKeepAll == false : call PropagateTempListToDetector() --> photons absorbed are not kept
    //
    // if GROUND detector mode : check geometrical cuts
    // check if photon is a candidate : if not, it is lost
    //
    
    SinglePhoton* p = 0;
    for(UInt_t m=0; m < fTempList.size(); m++) {
        p = fTempList[m];
        if(p->Status() == LostByCut) SafeDelete(fTempList[m]);
    }
}


