#!/usr/bin/perl
# $Id: CreateNewRecoModuleClass.pl,v 1.3 2004/09/24 15:20:24 naumov Exp $
# simple tool to create an empty new class with right headers
# D. DeMarco, M. Pallavicini
# adopted for Reco Modules by Dmitry Naumov

use Getopt::Std;
getopt('npus');

use POSIX qw(strftime);
$now_string = strftime "%b, %e %Y", gmtime;


$singleton = $opt_s;
$classname = $opt_n;
$parent = $opt_p;

$classnamemodule = $classname;
$classname =$classname."Module" ;

$classname or die "CreateNewRecoModuleClass: no name defined\n\t usage: CreateNewRecoModuleClass.pl -n name [-s singleton] [-u username] [-p parent]";

$uid = $<;
$user = (getpwuid($uid))[6];
$user=~s/,.*$//;

if ($opt_u) {
  $user = $opt_u;
}

$aa = $classname;
$aa=~y/[a-z]/[A-Z]/;

$ifd = "__".$aa."_HH_";
$inc = "";
if ( $parent ) {
  $inc = "\#include \"$parent.hh\"";
  $classparent = " : public $parent";
}

print "creating $classname.hh\n";
open(HH,">include/$classname.hh");

print HH <<pippo;
// ESAF : Euso Simulation and Analysis Framework
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

\#ifndef $ifd
\#define $ifd

\#include "euso.hh"
#include "RecoModule.hh"

class RecoEvent;
class RecoRootEvent;
$inc
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// $classname                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class $classname$classparent : public RecoModule  {
public:
    // ctor
    $classname();
    // dtor
    virtual ~$classname();

    // Init method: called at the beginning of a run
    Bool_t Init();

    // called before each event process
    Bool_t PreProcess();

    // event processing
    Bool_t Process( RecoEvent* );

    // called after processing an event
    Bool_t PostProcess();

    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );

    // called at the end of each run
    Bool_t Done();

    // memory clean
    void UserMemoryClean();
 
private:

    ClassDef($classname,0)
};

\#endif  /* $ifd */

pippo

close(HH);

print "creating $classname.cc\n";
open(CC,">src/$classname.cc");
print CC <<pippo;
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// $classname
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

\#include "$classname.hh"

ClassImp($classname)

//_____________________________________________________________________________
$classname\:\:$classname() : RecoModule("$classnamemodule") {
    // ctor

}

//_____________________________________________________________________________
$classname\:\:~$classname() {
    // dtor
}

//_____________________________________________________________________________
Bool_t $classname\:\:Init() {
     return kTRUE;
}

//_____________________________________________________________________________
Bool_t $classname\:\:PreProcess() {
     return kTRUE;
}

//_____________________________________________________________________________
Bool_t $classname\:\:Process(RecoEvent *ev) {
     return kTRUE;
}

//_____________________________________________________________________________
Bool_t $classname\:\:PostProcess() {
     return kTRUE;
}

//_____________________________________________________________________________
Bool_t $classname\:\:Done() {
    cout << "$classname done. " << endl;
     return kTRUE;
}

void $classname\:\:UserMemoryClean()  {

}

//_____________________________________________________________________________
Bool_t $classname\:\:SaveRootData(RecoRootEvent *fRecoRootEvent) {
    return kTRUE;
}
pippo

close(CC);


