cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c     Empirical formula for nucleus-nucleus inelastic cross section    c
c                                                                      c
c                                                                      c
c     REFERENCE: M.Kh.Anikina,G.L.Vardenga, A.I.Golokhvastov, et al.,  c
c                "Cross section for inelastic interaction of neon      c
c                nuclei with various other nuclei", YaF 39, No.1(7)    c      
c                (1983) 41-44 [Sov.J.Nucl. Phys. 38, No.1 (July 1983)  c
c                88-89].                                               c
c                                                                      c
c     NOTE 1: in the original paper formula R0=1.3F, S_A=10*pi*R0**2.  c
c             And this is a puzzle... Let's remind that 1 F (Fermi) =  c
c             1 fm (femtometre) = 10^{-15} m.                          c
c     NOTE 2: AN,AO,... are the atomic weights of N,O,...              c
c             CN,CO,... are the concentrations of N,O,...              c
c             SN,SO,... are pN,pO,... inelastic cross sections [barn]  c
c             A is the atomic weight of the projectile nucleus.        c
c     NOTE 3: The right recipe for averaging cross sections over the   c
c             nuclear composition of the air is not quite obvious.     c
c             However the two alternatives give very close results.    c
c                                                                      c
c    AUTHOR:   V.A. Naumov                                             c 
c    Modified: D.V. Naumov                                             c
c    DATE:   26.09.2002                                                c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
************************************************************************
      DOUBLE PRECISION FUNCTION IntLen(A,E) ! Nuclear interaction length           
************************************************************************
      IMPLICIT NONE
      REAL*8 R0,S_A,q,S_P,F_A1,N_A,F_A2,Aair,F_P1,F_P2,AN,AO,AA,AC
      REAL*8 CN,CO,CA,CC,SN,SO,SA,SC,Sp1,Lp1,Sp2,Lp2,PI
      REAL*8 A,E,B,S,Y,X, One, Third, NUCLEON_AIR_CROSS_SECTION
      DATA N_A/6.02D23/,Aair/14.483/,One/1.d0/,R0/1.287d-13/
      DATA S_P/1.d-24/,q/0.930/,AN/14.00674/,AO/15.99940/,AA/39.94800/
      DATA AC/12.01100/,CN/0.78470/,CO/0.21050/,CA/0.00470/,CC/0.00010/
      DATA SN/0.26500/,SO/0.29200/,SA/0.56600/,SC/0.23100/,PI/3.1415926/
*     
      Third = 1.D0/3
      S_A = pi*R0**2
      F_A1= One/(S_A*N_A)
      F_A2= Aair*F_A1
      F_P1= One/(S_P*N_A)
      F_P2= Aair*F_P1
      Sp1 = CN*SN/AN+CO*SO/AO+CA*SA/AA+CC*SC/AC
      Lp1 = F_P1/Sp1
      Sp2 = CN*SN   +CO*SO   +CA*SA   +CC*SC
      Lp2 = F_P2/Sp2
*
      X   = A**Third
      Y   = One/X
c  B=AN**Third; S=(CN/AN)*(B+X-q*(One/B+Y))**2     ! Nitrogen (N2)
c  B=AO**Third; S=(CO/AO)*(B+X-q*(One/B+Y))**2+S   ! Oxigen (O2)
c  B=AA**Third; S=(CA/AA)*(B+X-q*(One/B+Y))**2+S   ! Argon (Ar)
c  B=AC**Third; S=(CC/AC)*(B+X-q*(One/B+Y))**2+S   ! Carbon (in CO2)
c  IntLen_A=F_A1/S                                 ! [g/cm^2]

      B = AN**Third
      S = CN*(B+X-q*(One/B+Y))**2 ! Nitrogen (N2)
      B = AO**Third
      S = CO*(B+X-q*(One/B+Y))**2+S ! Oxigen (O2)
      B = AA**Third
      S = CA*(B+X-q*(One/B+Y))**2+S ! Argon (Ar)
      B = AC**Third
      S = CC*(B+X-q*(One/B+Y))**2+S ! Carbon (in CO2)
*
      IntLen = F_A2/S*NUCLEON_AIR_CROSS_SECTION(1.D11/A)/
     +     NUCLEON_AIR_CROSS_SECTION(E/A) ! [g/cm^2]
      END 

*     ... DOUBLE PRECISION FUNCTION NUCLEON_AIR_CROSS_SECTION
*     ... USES AN EMPIRICAL PARAMETRIZATION OF TOTAL INELASTIC 
*     ... NUCLEON AIR CROSS SECTION (in mbarn)
*     ... INPUT:  NUCLEON ENERGY in eV
*     ... OUTPUT: NUCLEON AIR CROSS SECTION (in mbarn)
*     ... REFERENCES: 
*     ... 1. Mielke H.H., Foller, M, Knapp J. 
*     ...    // J.Phys.G: Nucl.Part.Phys. 1994. V 20, P637
*     ... 2. V.A. Naumov, T.S. Sinegovskaya (eq (17))
*     ... AUTHOR: D.V. Naumov
*     ... DATE:   26.09.2002    
      DOUBLE PRECISION FUNCTION NUCLEON_AIR_CROSS_SECTION(E)
      IMPLICIT NONE
      REAL*8 S0,S1,S2,Estar,E1,E,THETA_FUNCTION
      DATA S0/290.0/,S1/8.7/,S2/1.14/,Estar/45.4D9/,E1/1.D9/
*
      NUCLEON_AIR_CROSS_SECTION = S0 - THETA_FUNCTION(E-Estar)*
     +     (S1*LOG(E/E1) - S2*LOG(E/E1)**2)
      END

      DOUBLE PRECISION FUNCTION THETA_FUNCTION(X)
      IMPLICIT NONE
      REAL*8 X
      IF(X.LT.0) THETA_FUNCTION = 0.D0
      IF(X.GE.0) THETA_FUNCTION = 1.D0
      END

